////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "SetupInfo.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "strfunc.h"
#include <sys/stat.h>

#define SETUP_FUNC  "psiphon.cnf"

#define _T(X) X

CSetupInfo g_info;

CSetupInfo::CSetupInfo()
{
	Init();
}

CSetupInfo::CSetupInfo(const char* sIn)
{
	Init();
	Parse(sIn);
}

CSetupInfo::~CSetupInfo()
{
	delete m_sIP;
	delete m_sName;
	delete m_Common;					
	delete m_Country;				
	delete m_State;					
	delete m_Email;					
	delete m_Unit;					
	delete m_Organization;
	delete m_sIPServer;
	delete m_sFinger;
}

void CSetupInfo::Init()
{
	m_sIP = strdup(_T("127.0.0.1"));
	m_nPort = 443;
	m_nShowImages = 1;
	m_nAutoUpdateIP = 1;
	m_nTestPort = 1;
	m_nStartOnStartUp = 0;
	m_sName = NULL;
	m_nCheckForUpdatesOnStartUp = 1;
	m_Common = strdup(_T("somename.somewhere.com"));
	m_Country = strdup(_T("US"));
	m_State = strdup(_T("Some State"));
	m_Email = strdup(_T("root@somename.somewhere.com"));
	m_Unit = strdup(_T("My Subunit of Large Organization"));
	m_Organization = strdup(_T("My Large Organization Name"));
	m_nRegenrateOnStartUp = 0;
	m_nLogLevel = 0;
	m_sIPServer = strdup(_T("http://psiphon.civisec.org/ip.php"));
	m_sFinger = NULL;
}

void CSetupInfo::Out(char*& sOut)
{
	char FORMAT[] = _T("%s\t%d\t%d\t%d\t%d\t%d\t%s\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%d\t%d\t%s");
	
	int nSize = GetStrLength(FORMAT)
			  + GetStrLength(m_sIP)
			  + GetStrLength(m_nPort)
			  + GetStrLength(m_nShowImages)
			  + GetStrLength(m_nAutoUpdateIP)
			  + GetStrLength(m_nTestPort)
			  + GetStrLength(m_nStartOnStartUp)
			  + GetStrLength(m_sName)
			  + GetStrLength(m_nCheckForUpdatesOnStartUp)
			  + GetStrLength(m_Common)
			  + GetStrLength(m_Country)
			  + GetStrLength(m_State)
			  + GetStrLength(m_Email)
			  + GetStrLength(m_Organization)
			  + GetStrLength(m_Unit)
			  + GetStrLength(m_nRegenrateOnStartUp)
			  + GetStrLength(m_nLogLevel)
			  + GetStrLength(m_sIPServer);
	
	sOut = new char[nSize];

	int nLen = _snprintf(sOut, nSize, FORMAT
										, m_sIP
										, m_nPort
										, m_nShowImages
										, m_nAutoUpdateIP
										, m_nTestPort
										, m_nStartOnStartUp
										, m_sName
										, m_nCheckForUpdatesOnStartUp
										, m_Common
										, m_Country
										, m_State
										, m_Email
										, m_Organization
										, m_Unit
										, m_nRegenrateOnStartUp
										, m_nLogLevel
										, m_sIPServer);

	if(nLen <= 0 || nLen >= nSize)
	{
		delete sOut;
		sOut = NULL;
	}
}

void CSetupInfo::Parse(const char* sIn)
{
	if(!sIn || !*sIn)
		return;

	char* _sIn = strdup(sIn);

	char seps[] = _T("\t");
	char* token = strtok(_sIn, seps);

	if(!token)
		return; 

	if(strcmp(token, _T("NULL")) != 0)
		SetIP(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetPort(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetShowImages(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetAutoUpdateIP(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetTestPort(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetStartOnStartUp(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetName(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetCheckForUpdatesOnStartUp(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetCommon(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetCountry(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetState(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetEmail(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetOrganization(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetUnit(token);

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetRegenrateOnStartUp(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetLogLevel(atoi(token));

	token = strtok(NULL, seps);
	if(!token)
	{
		delete _sIn;
		return;
	}
	SetIPServer(token);

	delete _sIn;
}

bool CSetupInfo::Store()
{
	FILE* pFile = fopen(SETUP_FUNC, _T("w"));
	if(pFile == NULL)
		return false;

	char* pOut = NULL;
	Out(pOut);
	if(pOut == NULL)
		return false;

	fwrite(pOut, sizeof(char), strlen(pOut), pFile);

	delete pOut;

	fclose(pFile);

	return true;
}

void CSetupInfo::Load()
{
	struct stat results;
	if(stat(SETUP_FUNC, &results) != 0)
		return;

	FILE* pFile = fopen(SETUP_FUNC, _T("r"));
	if(pFile == NULL)
		return;

	int nSize = results.st_size/sizeof(char) + 1;

	char* pIn = new char[nSize];
	memset(pIn, 0, nSize * sizeof(char));

	fread(pIn, sizeof(char), nSize, pFile);

	pIn[nSize - 1] = 0;

	Parse(pIn);

	delete pIn;

	fclose(pFile);
}

char* CSetupInfo::GetIP()
{
	return m_sIP;
}

void CSetupInfo::SetIP(const char* _sIP)
{
	delete 	m_sIP;
	m_sIP = strdup(_sIP);
}

int CSetupInfo::GetPort()
{
	return m_nPort;
}

void CSetupInfo::SetPort(int _nPort)
{
	m_nPort = _nPort;
}

int CSetupInfo::GetShowImages()
{
	return m_nShowImages;
}

void CSetupInfo::SetShowImages(int _nShowImages)
{
	m_nShowImages = _nShowImages;
}

int CSetupInfo::GetAutoUpdateIP()
{
	return m_nAutoUpdateIP;
}

void CSetupInfo::SetAutoUpdateIP(int _nAutoUpdateIP)
{
	m_nAutoUpdateIP = _nAutoUpdateIP;
}

int CSetupInfo::GetTestPort()
{
	return m_nTestPort;
}

void CSetupInfo::SetTestPort(int _nTestPort)
{
	m_nTestPort = _nTestPort;
}

int CSetupInfo::GetStartOnStartUp()
{
	return m_nStartOnStartUp;
}

void CSetupInfo::SetStartOnStartUp(int _nStartOnStartUp)
{
	m_nStartOnStartUp = _nStartOnStartUp;
}

char* CSetupInfo::GetName()
{
	return m_sName;
}

void CSetupInfo::SetName(const char* _sName)
{
	delete 	m_sName;
	m_sName = strdup(_sName);
}

int CSetupInfo::GetCheckForUpdatesOnStartUp()
{
	return m_nCheckForUpdatesOnStartUp;
}

void CSetupInfo::SetCheckForUpdatesOnStartUp(int _nCheckForUpdatesOnStartUp)
{
	m_nCheckForUpdatesOnStartUp = _nCheckForUpdatesOnStartUp;
}

void CSetupInfo::SetCommon(const char* _Common)
{
	delete 	m_Common;
	m_Common = strdup(_Common);
}

void CSetupInfo::SetUnit(const char* _Unit)
{
	delete 	m_Unit;
	m_Unit = strdup(_Unit);
}

void CSetupInfo::SetOrganization(const char* _Organization)
{
	delete 	m_Organization;
	m_Organization = strdup(_Organization);
}

void CSetupInfo::SetEmail(const char* _Email)
{
	delete 	m_Email;
	m_Email = strdup(_Email);
}

void CSetupInfo::SetCountry(const char* _Country)
{
	delete 	m_Country;
	m_Country = strdup(_Country);
}

void CSetupInfo::SetState(const char* _State)
{
	delete 	m_State;
	m_State = strdup(_State);
}

void CSetupInfo::SetRegenrateOnStartUp(int _nRegenrateOnStartUp)
{
	m_nRegenrateOnStartUp = _nRegenrateOnStartUp;
}

char* CSetupInfo::GetCommon()
{
	return 	m_Common;
}

char* CSetupInfo::GetUnit()
{
	return 	m_Unit;
}

char* CSetupInfo::GetOrganization()
{
	return 	m_Organization;
}

char* CSetupInfo::GetEmail()
{
	return 	m_Email;
}

char* CSetupInfo::GetCountry()
{
	return 	m_Country;
}

char* CSetupInfo::GetState()
{
	return 	m_State;
}

int CSetupInfo::GetRegenrateOnStartUp()
{
	return m_nRegenrateOnStartUp;
}

int CSetupInfo::GetLogLevel()
{
	return m_nLogLevel;
}

void CSetupInfo::SetLogLevel(int _nLogLevel)
{
	m_nLogLevel = _nLogLevel;
}

char* CSetupInfo::GetIPServer()
{
	return m_sIPServer;
}

void CSetupInfo::SetIPServer(const char* _sIPServer)
{
	delete 	m_sIPServer;
	m_sIPServer = strdup(_sIPServer);
}

char* CSetupInfo::GetFinger()
{
	return m_sFinger;
}

void CSetupInfo::SetFinger(const char* _sFinger)
{
	delete 	m_sFinger;
	m_sFinger = strdup(_sFinger);
}

