////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#pragma once

typedef void (*LOG_ROUTINE)(const char* msg);

class CDBUser
{
	static LOG_ROUTINE	m_pLogFunc; 
	static char* m_sDB;

private:
	int		m_nID;
	char* 	m_sUserName;
	char* 	m_sPassword;
	char* 	m_sEmail;
	char* 	m_sFullName;
	bool	m_nDeleted;

private:
	int	m_nLastAccess;
	int	m_nWritten;
	int	m_nWriteTime;
	int	m_nRead;
	int	m_nReadTime;

public:
	CDBUser();
	~CDBUser();
	
	void	Init();

	int		GetID();
	char*	GetUsername();
	char*	GetPassword();
	char*	GetEmail();
	char*	GetFullName();
	bool	IsDeleted();

	int		GetLastAccess();
	int		GetWritten();
	int		GetRead();
	int		GetWriteTime();
	int		GetReadTime();

	void	SetID(int _nID);
	void	SetUsername(const char* _s);
	void	SetPassword(const char* _s);
	void	SetEmail(const char* _s);
	void	SetFullName(const char* _s);
	void	SetDeleted(bool b);

	void	SetLastAccess(int _nLastAccess);
	void	SetWritten(int _nWritten);
	void	SetRead(int _nRead);
	void	SetWriteTime(int _nWriteTime);
	void	SetReadTime(int _nReadTime);

	int		Insert();
	int		Get(int nUserID);
	int		Update();

	static int	ResetPassword(int nUserID, const char* sPwd);
	static int	FindUser(const char* pUserName, const char* pPassword);
	static int	FindUsername(const char* pUserName);
	static int	FindUserName(const char* pUserName);
	static int	Delete(int nUserID);
	static int	GetUserCount();
	static int	LoadUsers(CDBUser**& pUsers);

	static void	EncryptPassword(char* pKey, int nKeySize, const char* _pUsername, const char* _pPassword);

	static void	SetLogFunc(LOG_ROUTINE m_pLogFunc);
	static void	Log(const char* msg);

protected:
	void Cleanup();
};
