////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "bookmarks.h"
#include "CppSQLite3.h"
#include "convert.h"
#include "appWeb/appWeb.h"
#include "strfunc.h"

#define _T(X) X

char* CDBUserBookmarks::m_sDB = _T("bookmarks.db");

int CDBUserBookmarks::AddDBBookMark(char* sURL, int nUserID)
{
	if(nUserID <= 0)
		return -1;

	if(sURL == NULL)
		return -1;

	int nURLLen = strlen(sURL);
	if(nURLLen <= 0 || nURLLen >= 255)
		return -1;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("bookmarks")))
		{
			db.execDML(_T("create table bookmarks(nIndex integer primary key, nUserID int, sURL char(255));"));
			db.execDML(_T("create unique index idx_bookmark_user on bookmarks(nUserID ASC, sURL ASC);"));
		}

		CppSQLite3Statement stmt = db.compileStatement(_T("insert into bookmarks(nUserID, sURL)"
		" values(?, ?);"));		

		stmt.bind(1, nUserID); 
		stmt.bind(2, sURL); 

		stmt.execDML();

		return 0;
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserBookmarks::AddDBBookMark"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return -1;
}

int CDBUserBookmarks::DeleteDBBookMark(char* sURL, int nUserID)
{
	if(nUserID <= 0)
		return -1;

	if(sURL == NULL)
		return -1;

	int nURLLen = strlen(sURL);
	if(nURLLen <= 0 || nURLLen >= 255)
		return -1;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("bookmarks")))
			return 0;

		CppSQLite3Statement stmt = db.compileStatement(_T("delete from bookmarks where nUserID=?"
		" and sURL=?;"));		

		stmt.bind(1, nUserID);
		stmt.bind(2, sURL);
		stmt.execDML();


		return 0;
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserBookmarks::DeleteDBBookMark"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return -1;
}

int CDBUserBookmarks::DeleteDBBookMarks(int nUserID)
{
	if(nUserID <= 0)
		return -1;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("bookmarks")))
			return 0;

		CppSQLite3Statement stmt = db.compileStatement(_T("delete from bookmarks where nUserID=?;"));		

		stmt.bind(1, nUserID);
		stmt.execDML();
		return 0;
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserBookmarks::DeleteDBBookMarks"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return -1;
}

char* CDBUserBookmarks::BuildDBBookMarks(int nUserID)
{
	if(nUserID <= 0)
		return NULL;

	try
	{
		char* sBookMarksOutput = NULL;

		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("bookmarks")))
			return NULL;

		CppSQLite3Statement stmt = db.compileStatement(_T("select sURL from bookmarks "
														" where nUserID=? order by sURL ASC;"));

		stmt.bind(1, nUserID);

		CppSQLite3Query q = stmt.execQuery();

		while(!q.eof())
		{
			const char* sURL = (q.getStringField(0));
			if(sURL == NULL)
			{
				delete sBookMarksOutput;
				return NULL;
			}

			int nURLLen = strlen(sURL);
			if(nURLLen <= 0 || nURLLen >= 255)
			{
				delete sBookMarksOutput;
				return NULL;
			}

			int bufLen = 0;

			if(sBookMarksOutput)
				bufLen += strlen(sBookMarksOutput);

			bufLen += strlen(sURL) + strlen(_T("\n")) + 1;

			char* pBookMarksBuf = new char[bufLen];

			int nLen = 0;

			if(sBookMarksOutput)
				nLen = _snprintf(pBookMarksBuf, bufLen, _T("%s%s\n"), sBookMarksOutput, sURL);
			else
				nLen = _snprintf(pBookMarksBuf, bufLen, _T("%s\n"), sURL);

			if(nLen <= 0 || nLen >= bufLen)
			{
				delete pBookMarksBuf;
				return NULL;
			}

			delete sBookMarksOutput;
			sBookMarksOutput = pBookMarksBuf;

			q.nextRow();
		}

		return sBookMarksOutput;
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserBookmarks::BuildDBBookMarks"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return NULL;
}
