////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "convert.h"
#include <string.h>
#include <stdlib.h>
#include <wchar.h>

CConverter::CConverter(const char* pA)
{
	Init();

	if(pA == NULL)
		return;

	int nSize = strlen(pA);
	nSize++;

	m_pW = new wchar_t[nSize];
	mbstowcs(m_pW, pA, nSize);
	m_pW[nSize - 1] = 0;

	m_pA = new char[nSize];
	strncpy(m_pA, pA, nSize);
}

CConverter::CConverter(const wchar_t* pW)
{
	Init();

	if(pW == NULL)
		return;

	int nSize = wcslen(pW);
	nSize++;

	m_pA = new char[nSize];
	wcstombs(m_pA, pW, nSize);
	m_pA[nSize - 1] = 0;

	m_pW = new wchar_t[nSize];
	wcsncpy(m_pW, pW, nSize);
}

void CConverter::Init()
{
	m_pW = NULL;
	m_pA = NULL;
}

CConverter::~CConverter()
{
	delete m_pW;
	delete m_pA;
}

CConverter::operator const char*()
{
	return GetA();
}

CConverter::operator const wchar_t*()
{
	return GetW();
}

const wchar_t* CConverter::GetW()
{
	return m_pW;
}

const char* CConverter::GetA()
{
	return m_pA;
}

void CConverter::Escape()
{
	escapeA(m_pA);	
	escapeW(m_pW);
}

void CConverter::escapeA(char*& pA)
{
	if(pA == NULL)
		return;

	int nSize = strlen(pA);
	nSize++;

	int nCnt = 0;
	int i = 0;
	for(i = 0;i < nSize;i++)
	{
		if(pA[i] == '\'')
			nCnt++;
	}

	char* pDest = new char[nSize + nCnt];

	int j = 0;
	for(i = 0;i < nSize;i++)
	{
		if(pA[i] != '\'')
		{
			pDest[j] = pA[i];
		}
		else
		{
			pDest[j] = '\'';
			j++;
			pDest[j] = '\'';
		}

		j++;
	}

	delete pA;
	pA = pDest;
}

#define W(x)	  L ## x

void CConverter::escapeW(wchar_t*& pW)
{
	if(pW == NULL)
		return;

	int nSize = wcslen(pW);
	nSize++;

	int nCnt = 0;
	int i = 0;
	for(i = 0;i < nSize;i++)
	{
		if(pW[i] == W('\''))
			nCnt++;
	}

	wchar_t* pDest = new wchar_t[nSize + nCnt];

	int j = 0;
	for(i = 0;i < nSize;i++)
	{
		if(pW[i] != W('\''))
		{
			pDest[j] = pW[i];
		}
		else
		{
			pDest[j] = W('\'');
			j++;
			pDest[j] = W('\'');
		}

		j++;
	}

	delete pW;
	pW = pDest;
}
