////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "appWeb/appWeb.h"
#include "cookies.h"
#include "CppSQLite3.h"
#include "strfunc.h"

#define _T(X) X

char* CDBUserCookies::m_sDB = _T("user_cookies.db");

static const char haystack[37] = _T("janfebmaraprmayjunjulaugsepoctnovdec");

struct Zone
{
	const char* tzName;
	int			tzOffset;
};

static const Zone known_zones[] =
{
	{ _T("UT"), 0 },
	{ _T("GMT"), 0 },
	{ _T("EST"), -300 },
	{ _T("EDT"), -240 },
	{ _T("CST"), -360 },
	{ _T("CDT"), -300 },
	{ _T("MST"), -420 },
	{ _T("MDT"), -360 },
	{ _T("PST"), -480 },
	{ _T("PDT"), -420 },
	{ 0, 0 }
};

unsigned int ymdhms_to_seconds(int year, int mon, int day, int hour, int minute, int second)
{
	if(sizeof(time_t) == 4)
	{
		if((time_t) - 1 < 0)
		{
			if(year >= 2038)
			{
				year = 2038;
				mon = 0;
				day = 1;
				hour = 0;
				minute = 0;
				second = 0;
			}
		}
		else
		{
			if(year >= 2115)
			{
				year = 2115;
				mon = 0;
				day = 1;
				hour = 0;
				minute = 0;
				second = 0;
			}
		}
	}

	unsigned int ret = (day - 32075)	   /* days */
	+1461L * (year + 4800L + (mon - 14) / 12) / 4 + 367 * (mon - 2 - (mon - 14) / 12 * 12) / 12 - 3 * ((year + 4900L + (mon - 14) / 12) / 100) / 4 - 2440588;
	
	ret = 24 * ret + hour;     /* hours   */
	ret = 60 * ret + minute;   /* minutes */
	ret = 60 * ret + second;   /* seconds */

	return ret;
}

CCookie::CCookie()
{
	Init();
}

CCookie::~CCookie()
{
	Reset();
}

void CCookie::Init()
{
	m_pCookieDomain = NULL; 
	m_pCookieName = NULL; 
	m_pCookieVal = NULL; 
	m_pCookiePath = NULL; 
	m_pCookieExpires = NULL;
}

void CCookie::Reset()
{
	delete m_pCookieDomain;
	delete m_pCookieName;
	delete m_pCookieVal;
	delete m_pCookiePath;
	delete m_pCookieExpires;
	Init();
}

void CCookie::Setup(char* raw_cookie, const CParsedUrl& reqURL)
{

	Reset();

	char* details = strdup(raw_cookie);

	char* key = details;
	char* tok = NULL;
	char* keyValue = NULL;
	char* dp = NULL;
	char* sp = NULL;

	int seenSemi = 0;
	int seenValue = 0;

	while(*key)
	{
		while(*key && isspace(*key))
		{
			key++;
		}
		
		tok = key;

		while(*tok && !isspace(*tok) && *tok != ';' && *tok != '=')
		{
			tok++;
		}

		if(*tok)
		{
			*tok++ = '\0';
		}

		while(isspace(*tok))
		{
			tok++;
		}

		seenSemi = 0;

		if(*tok == '\"')
		{
			keyValue = ++tok;

			while(*tok != '\"' && *tok != '\0')
			{
				tok++;
			}

			if(*tok)
			{
				*tok++ = '\0';
			}
		}
		else
		{
			keyValue = tok;

			while(*tok != ';' && *tok != '\0')
			{
				tok++;
			}

			if(*tok)
			{
				seenSemi++;
				*tok++ = '\0';
			}
		}

		//
		//	Handle back-quoting
		//
		if(strchr(keyValue, '\\'))
		{
			for(dp = sp = keyValue;*sp;sp++)
			{
				if(*sp == '\\')
				{
					sp++;
				}
				*dp++ = *sp++;
			}
			*dp = '\0';
		}

		if(!seenValue)
		{
			SetCookieName(key);
			SetCookieVal(keyValue);

			seenValue++;
		}
		else
		{
			switch(tolower(*key))
			{
			case 'd':
				if(this && stricmp(key, _T("domain")) == 0)
				{
					SetCookieDomain(keyValue);
				}
				break;
			case 'p':
				if(this && stricmp(key, _T("path")) == 0)
				{
					SetCookiePath(keyValue);
				}
				break;
			case 'e':
				if(this && stricmp(key, _T("expires")) == 0)
				{
					SetCookieExpires(keyValue);
				}
				break;

			default:
				//	Just ignore keywords we don't understand
				;
			}
		}

		key = tok;

		if(!seenSemi)
		{
			while(*key && *key != ';')
			{
				key++;
			}
			if(*key)
			{
				key++;
			}
		}
	}

	if(!GetCookieDomain())
		SetCookieDomain(reqURL.GetHost());

	if(!GetCookiePath())
		SetCookiePath(reqURL.GetPath());

	if(!GetCookieVal())
		SetCookieVal("");

	if(!GetCookieExpires())
		SetCookieExpires(""); //session cookie

	delete details;
}

void CCookie::SetCookieDomain(const char* domain)
{
	delete m_pCookieDomain;
	m_pCookieDomain = strdup(domain);
}

char* CCookie::GetCookieDomain(void)
{
	return m_pCookieDomain;
}

void CCookie::SetCookieName(const char* name)
{
	delete m_pCookieName;
	m_pCookieName = strdup(name);
}

char* CCookie::GetCookieName(void)
{
	return m_pCookieName;
}

void CCookie::SetCookieVal(const char* value)
{
	delete m_pCookieVal;
	m_pCookieVal = strdup(value);
}

char* CCookie::GetCookieVal(void)
{
	return m_pCookieVal;
}

void CCookie::SetCookiePath(const char* path)
{
	delete m_pCookiePath;
	m_pCookiePath = strdup(path);
}

char* CCookie::GetCookiePath(void)
{
	return m_pCookiePath;
}

void CCookie::SetCookieExpires(const char* expires)
{
	delete m_pCookieExpires;
	m_pCookieExpires = strdup(expires);
}

char* CCookie::GetCookieExpires(void)
{
	return m_pCookieExpires;
}

//////////////////////////////////////////////////////////////////////////////////////////////////

int CDBUserCookies::CheckDomainMatch(const char* sCookieDomain, const char* Host)
{
	if(sCookieDomain == NULL)
		return 0;

	if(Host == NULL)
		return 0;

	/* Numeric address requires exact match.  It also requires HOST to
		be an IP address.  */
	if(IsNumericAddress(sCookieDomain))
		return (0 == strcmp(sCookieDomain, Host));

	/* For the sake of efficiency, check for exact match first. */
	if(0 == stricmp(sCookieDomain, Host))
		return 1;

	/* HOST must match the tail of cookie_domain. */
	if(!MatchTail(Host, sCookieDomain, 1))
		return 0;

	/* We know that COOKIE_DOMAIN is a subset of HOST; however, we must
	   make sure that somebody is not trying to set the cookie for a
	   subdomain shared by many entities.  For example, "company.co.uk"
	   must not be allowed to set a cookie for ".co.uk".  On the other
	   hand, "sso.redhat.de" should be able to set a cookie for
	   ".redhat.de".
	   The only marginally sane way to handle this I can think of is to
	   reject on the basis of the length of the second-level domain name
	   (but when the top-level domain is unknown), with the assumption
	   that those of three or less characters could be reserved.  For
	   example:
			.co.org -> works because the TLD is known
			 .co.uk -> doesn't work because "co" is only two chars long
			.com.au -> doesn't work because "com" is only 3 chars long
			.cnn.uk -> doesn't work because "cnn" is also only 3 chars long (ugh)
			.cnn.de -> doesn't work for the same reason (ugh!!)
		   .abcd.de -> works because "abcd" is 4 chars long
		.img.cnn.de -> works because it's not trying to set the 2nd level domain
		 .cnn.co.uk -> works for the same reason
	  That should prevent misuse, while allowing reasonable usage.  If
	  someone knows of a better way to handle this, please let me
	  know.  */
	{
		const char* p = sCookieDomain;
		int dccount = 1;		/* number of domain components */
		int ldcl = 0;		/* last domain component length */
		int nldcl = 0;		/* next to last domain component length */
		int out = 0;
		if(*p == '.')
		/* Ignore leading period in this calculation. */
			++p;

		for(out = 0;!out;p++)
			switch(*p)
			{
			case '\0':
				out = 1;
				break;
			case '.':
				if(ldcl == 0)
				/* Empty domain component found -- the domain is invalid. */
					return 0;
				if(*(p + 1) == '\0')
				{
					/* Tolerate trailing '.' by not treating the domain as
																																																					 one ending with an empty domain component.  */
					out = 1;
					break;
				}
				nldcl = ldcl;
				ldcl = 0;
				++dccount;
				break;
			default:
				++ldcl;
			}

		if(dccount < 2)
			return 0;

		if(dccount == 2)
		{
			int i = 0;
			int known_toplevel = 0;
			static const char* known_toplevel_domains[] =
			{
				_T(".com"), _T(".edu"), _T(".net"), _T(".org"), _T(".gov"), _T(".mil"), _T(".int")
			};
			for(i = 0;i < (sizeof(known_toplevel_domains) / sizeof((known_toplevel_domains)[0]));i++)
				if(MatchTail(sCookieDomain, known_toplevel_domains[i], 1))
				{
					known_toplevel = 1;
					break;
				}
			if(!known_toplevel && nldcl <= 3)
				return 0;
		}
	}

	/* Don't allow the host "foobar.com" to set a cookie for domain
	   "bar.com".  */
	if(*sCookieDomain != '.')
	{
		int dlen = strlen(sCookieDomain);
		int hlen = strlen(Host);
		/* cookie host:    hostname.foobar.com */
		/* desired domain:  		   bar.com */
		/* '.' must be here in host-> ^ 	   */
		if(hlen > dlen && Host[hlen - dlen - 1] != '.')
			return 0;
	}

	return 1;
}

int CDBUserCookies::MatchTail(const char* sString, const char* sTail, int nCaseSensitive)
{
	int i = 0;
	int j = 0;

	if(!nCaseSensitive)
	{
		for(i = strlen(sString), j = strlen(sTail);i >= 0 && j >= 0;i--, j--)
		{
			if(sString[i] != sTail[j])
				break;
		}
	}
	else
	{
		for(i = strlen(sString), j = strlen(sTail);i >= 0 && j >= 0;i--, j--)
		{
			if(tolower(sString[i]) != tolower(sTail[j]))
				break;
		}
	}

	if(j == -1)
		return 1;
	else
		return 0;
}

bool REQUIRE_DIGITS(const char*& p)
{
	if(!isdigit(*p))
		return 0;					

	return 1;
}

bool REQUIRE_DOT(const char*& p)
{
	if(*p++ != '.')
		return 0;					

	return 1;
}

bool IsNumericAddress(const char* pAddr)
{
	const char* p = pAddr;

	REQUIRE_DIGITS(p);		/* A */
	REQUIRE_DOT(p);		/* . */
	REQUIRE_DIGITS(p);		/* B */
	REQUIRE_DOT(p);		/* . */
	REQUIRE_DIGITS(p);		/* C */
	REQUIRE_DOT(p);		/* . */
	REQUIRE_DIGITS(p);		/* D */

	if(*p != '\0')
		return false;

	return true;
}


time_t CDBUserCookies::ParseDate(const char* dateString)
{
	// This parse a date in the form:
	//     Wednesday, 09-Nov-99 23:12:40 GMT
	// or
	//     Sat, 01-Jan-2000 08:00:00 GMT
	// or
	//     Sat, 01 Jan 2000 08:00:00 GMT
	// or
	//     01 Jan 99 22:00 +0100	(exceptions in rfc822/rfc2822)
	//
	// We ignore the weekday
	//
	time_t result = 0;
	int offset = 0;
	char* newPosStr;
	int day = 0;
	char monthStr[4];
	int month = -1;
	int year = 0;
	int hour = 0;
	int minute = 0;
	int second = 0;

	// Strip leading space
	while(*dateString && isspace(*dateString))
		dateString++;

	// Strip weekday
	while(*dateString && !isdigit(*dateString) && !isspace(*dateString))
		dateString++;

	// Strip trailing space
	while(*dateString && isspace(*dateString))
		dateString++;

	if(!*dateString)
		return result;  // Invalid date

	if(isalpha(*dateString))
	{
		// ' Nov 5 1994 18:15:30 GMT'
		// Strip leading space
		while(*dateString && isspace(*dateString))
			dateString++;

		for(int i = 0;i < 3;i++)
		{
			if(!*dateString || (*dateString == '-') || isspace(*dateString))
				return result;  // Invalid date

			monthStr[i] = tolower(*dateString++);
		}
		monthStr[3] = '\0';

		newPosStr = (char*)strstr(haystack, monthStr);

		if(!newPosStr)
			return result;  // Invalid date

		month = (newPosStr - haystack) / 3; // Jan=00, Feb=01, Mar=02, ..

		if((month < 0) || (month > 11))
			return result;  // Invalid date

		while(*dateString && isalpha(*dateString))
			dateString++; // Skip rest of month-name
	}

	// ' 09-Nov-99 23:12:40 GMT'
	// ' 5 1994 18:15:30 GMT'
	day = strtol(dateString, &newPosStr, 10);
	dateString = newPosStr;

	if((day < 1) || (day > 31))
		return result; // Invalid date;

	if(!*dateString)
		return result;  // Invalid date

	while(*dateString && (isspace(*dateString) || (*dateString == '-')))
		dateString++;

	if(month == -1)
	{
		for(int i = 0;i < 3;i++)
		{
			if(!*dateString || (*dateString == '-') || isspace(*dateString))
				return result;  // Invalid date
			monthStr[i] = tolower(*dateString++);
		}
		monthStr[3] = '\0';

		newPosStr = (char*)strstr(haystack, monthStr);

		if(!newPosStr)
			return result;  // Invalid date

		month = (newPosStr - haystack) / 3; // Jan=00, Feb=01, Mar=02, ..

		if((month < 0) || (month > 11))
			return result;  // Invalid date

		while(*dateString && isalpha(*dateString))
			dateString++; // Skip rest of month-name
	}

	// '-99 23:12:40 GMT'
	while(*dateString && (isspace(*dateString) || (*dateString == '-')))
		dateString++;

	if(!*dateString || !isdigit(*dateString))
		return result;  // Invalid date

	// '99 23:12:40 GMT'
	year = strtol(dateString, &newPosStr, 10);
	dateString = newPosStr;

	// Y2K: Solve 2 digit years
	if((year >= 0) && (year < 50))
		year += 2000;

	if((year >= 50) && (year < 100))
		year += 1900;  // Y2K

	if((year < 1900) || (year > 2500))
		return result; // Invalid date

	// Don't fail if the time is missing.
	if(*dateString)
	{
		// ' 23:12:40 GMT'
		if(!isspace(*dateString++))
			return result;  // Invalid date

		hour = strtol(dateString, &newPosStr, 10);
		dateString = newPosStr;

		if((hour < 0) || (hour > 23))
			return result; // Invalid date

		if(!*dateString)
			return result;  // Invalid date

		// ':12:40 GMT'
		if(*dateString++ != ':')
			return result;  // Invalid date

		minute = strtol(dateString, &newPosStr, 10);
		dateString = newPosStr;

		if((minute < 0) || (minute > 59))
			return result; // Invalid date

		if(!*dateString)
			return result;  // Invalid date

		// ':40 GMT'
		if(*dateString != ':' && !isspace(*dateString))
			return result;  // Invalid date

		// seconds are optional in rfc822 + rfc2822
		if(*dateString == ':')
		{
			dateString++;

			second = strtol(dateString, &newPosStr, 10);
			dateString = newPosStr;

			if((second < 0) || (second > 59))
				return result; // Invalid date
		}
		else
		{
			dateString++;
		}

		while(*dateString && isspace(*dateString))
			dateString++;
	}

	// don't fail if the time zone is missing, some
	// broken mail-/news-clients omit the time zone
	if(*dateString)
	{
		if((strnicmp(dateString, _T("gmt"), 3) == 0) || (strnicmp(dateString, _T("utc"), 3) == 0))
		{
			dateString += 3;
			while(*dateString && isspace(*dateString))
				dateString++;
		}

		if((*dateString == '+') || (*dateString == '-'))
		{
			offset = strtol(dateString, &newPosStr, 10);
			if(abs(offset) < 30)
			{
				dateString = newPosStr;

				offset = offset * 100;

				if(*dateString && *(dateString + 1))
				{
					dateString++;
					int minutes = strtol(dateString, &newPosStr, 10);
					if(offset > 0)
						offset += minutes;
					else
						offset -= minutes;
				}
			}

			if((offset < -9959) || (offset > 9959))
				return result; // Invalid date

			int sgn = (offset < 0) ? -1 : 1;
			offset = abs(offset);
			offset = ((offset / 100) * 60 + (offset % 100)) * sgn;
		}
		else
		{
			for(int i = 0;known_zones[i].tzName != 0;i++)
			{
				if(0 == strnicmp(dateString, known_zones[i].tzName, strlen(known_zones[i].tzName)))
				{
					offset = known_zones[i].tzOffset;
					break;
				}
			}
		}
	}

	result = ymdhms_to_seconds(year, month + 1, day, hour, minute, second);

	// avoid negative time values
	if((offset > 0) && (offset > result))
		offset = 0;

	result -= offset * 60;

	// If epoch 0 return epoch +1 which is Thu, 01-Jan-70 00:00:01 GMT
	// This is so that parse error and valid epoch 0 return values won't
	// be the same for sensitive applications...
	if(result < 1)
		result = 1;

	return result;
}

void CDBUserCookies::StoreDBUserCookies(const CParsedUrl& reqURL, std::vector <CCookie*> cookiesAr, int nUserID)
{
	if(nUserID <= 0)
		return;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("user_cookies")))
		{
			db.execDML(_T("create table user_cookies(nIndex integer primary key, nUserID int, sDomain char(256),sPath char(256), sCookieName char(256), sCookieVal text, nCookieExpires int);"));
			db.execDML(_T("create index idx_cookie_user on user_cookies(nUserID ASC);"));
			db.execDML(_T("create index idx_cookie_domain on user_cookies(sDomain ASC);"));
			db.execDML(_T("create index idx_cookie_path on user_cookies(sPath ASC);"));
			db.execDML(_T("create index idx_cookie_name on user_cookies(sCookieName ASC);"));
		}

		CppSQLite3Statement update_stmt = db.compileStatement(_T("update user_cookies set sCookieVal =?, nCookieExpires =? where "
			"nUserID =? and sDomain =? and sPath =? and sCookieName =?;"));

		CppSQLite3Statement insert_stmt = db.compileStatement(_T("insert into user_cookies(nUserID, sDomain, sPath, sCookieName, sCookieVal, nCookieExpires)"
				"values(?, ?, ?, ?, ?, ?);"));

		db.execDML(_T("begin transaction;"));

		std::vector<CCookie*>::iterator itPos = cookiesAr.begin();
		for(;itPos < cookiesAr.end();itPos++)
		{
			if(CheckDomainMatch((*itPos)->GetCookieDomain(), reqURL.GetHost()) == 0) //cookie domain name doesn't match request domain
				continue;

			int nCookieExpires = ParseDate((*itPos)->GetCookieExpires());
			char* sCookieDomain = (*itPos)->GetCookieDomain();
			char* sCookieName = (*itPos)->GetCookieName();
			char* sCookieVal = (*itPos)->GetCookieVal();
			char* sCookiePath = (*itPos)->GetCookiePath();

			//check length, avoid truncation
			if(strlen(sCookieDomain) > 255)
				return;
			if(strlen(sCookieName) > 255)
				return;
			if(strlen(sCookiePath) > 255)
				return;

			//try to update first
			update_stmt.bind(1, sCookieVal);
			update_stmt.bind(2, nCookieExpires);
			update_stmt.bind(3, nUserID);
			update_stmt.bind(4, sCookieDomain);
			update_stmt.bind(5, sCookiePath);
			update_stmt.bind(6, sCookieName);
			int nRows = update_stmt.execDML();
			update_stmt.reset();

			if(nRows <= 0)
			{
				insert_stmt.bind(1, nUserID);
				insert_stmt.bind(2, sCookieDomain);
				insert_stmt.bind(3, sCookiePath);
				insert_stmt.bind(4, sCookieName);
				insert_stmt.bind(5, sCookieVal);
				insert_stmt.bind(6, nCookieExpires);
				insert_stmt.execDML();
				insert_stmt.reset();
			}
		}

		db.execDML(_T("commit transaction;"));
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserCookies::StoreDBSessionCookie"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}
}

char* CDBUserCookies::BuildCookies(const CParsedUrl& reqURL, int nUserID)
{
	if(nUserID <= 0)
		return NULL;

	int nCookieCount = 0;

	const char* s_tail = reqURL.GetHost();
	const char* s_path = reqURL.GetPath();

	if(!strlen(s_tail) || !strlen(s_path))
		return NULL;

	char* sCookieOutput = NULL;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists("user_cookies"))
			return NULL;

		while(s_tail)
		{
			int now = (int)time(NULL);

			CppSQLite3Statement stmt = db.compileStatement(_T("select sPath, sCookieName, sCookieVal from user_cookies "
			" where nUserID=? and sDomain =? and (nCookieExpires >? or nCookieExpires = '0');"));

			stmt.bind(1, nUserID);
			stmt.bind(2, s_tail);
			stmt.bind(3, now);

			CppSQLite3Query q = stmt.execQuery();

			while(!q.eof() && nCookieCount < 21) //20 - max num of cookies per domain
			{
				const char* sCookiePath = (q.getStringField(0));
				const char* sCookieName = (q.getStringField(1));
				const char* sCookieVal = (q.getStringField(2));

				if(strstr(s_path, sCookiePath) == s_path)
				{
					int bufLen = 0;

					if(sCookieOutput)
						bufLen += strlen(sCookieOutput);

					bufLen += strlen(sCookieName) + strlen(sCookieVal) + strlen(_T("=; ")) + 1;

					char* pCookieBuf = new char[bufLen];

					int nLen = 0;
					if(sCookieOutput)
						nLen = _snprintf(pCookieBuf, bufLen, _T("%s%s=%s; "), sCookieOutput, sCookieName, sCookieVal);
					else
						nLen = _snprintf(pCookieBuf, bufLen, _T("%s=%s; "), sCookieName, sCookieVal);

					if(nLen <= 0 || nLen >= bufLen)
					{
						delete pCookieBuf;
						return NULL;
					}

					delete sCookieOutput;
					sCookieOutput = pCookieBuf;

					nCookieCount++;
				}
				q.nextRow();
			}
			stmt.reset();
			s_tail = strchr(s_tail + 1, '.');
		}
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserCookies::BuildCookies"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return sCookieOutput;
}

void CDBUserCookies::PurgeDBUserCookies(int nUserID)
{
	if(nUserID <= 0)
		return;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("user_cookies")))
			return;

		CppSQLite3Statement stmt = db.compileStatement(_T("DELETE FROM user_cookies WHERE nUserID = ?;"));

		stmt.bind(1, nUserID);

		stmt.execDML();
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserCookies::PurgeDBSessionCookies"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}
}
