////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "logDetails.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "strfunc.h"

#ifndef min
#define min(a, b)  (((a) < (b)) ? (a) : (b))
#endif

#define _T(X) X

const char* PSIPHON_MESSAGE = _T("PSIPHON MESSAGE:");
const char* PSIPHON_MESSAGE_REQUEST = _T("request");

CPsiphonMessage::CPsiphonMessage()
{
	Init();
}

CPsiphonMessage::CPsiphonMessage(const char* sIn)
{
	Init();
	Parse(sIn);
}

CPsiphonMessage::~CPsiphonMessage()
{
	delete  m_sMessage;
	delete 	m_sIP;
	delete 	m_sURI;
	delete 	m_sOut;
}

void CPsiphonMessage::Init()
{
	m_sOut = NULL;

	m_sMessage = NULL;
	m_sIP = NULL;
	m_sURI = NULL;
	m_nUserID = 0;
	m_nLastAccess = 0;
	m_nWritten = 0;
	m_nRead = 0;
	m_nReadTime = 0;
}

void CPsiphonMessage::Out(char*& pOut)
{
	char FORMAT[] = _T("%s\t%s\t%s\t%s\t%d\t%d\t%d\t%d\t%d\t%d");

	int nSize = GetStrLength(FORMAT)
			  + GetStrLength(PSIPHON_MESSAGE)
			  + GetStrLength(m_sMessage)
			  + GetStrLength(m_sIP)
			  + GetStrLength(m_sURI)
			  + GetStrLength(m_nUserID)
			  + GetStrLength(m_nLastAccess)
			  + GetStrLength(m_nWritten)
			  + GetStrLength(m_nWriteTime)
			  + GetStrLength(m_nRead)
			  + GetStrLength(m_nReadTime)
			  + 1;

	delete pOut;
	pOut = new char[nSize];

	int nLen = _snprintf(pOut, nSize, FORMAT, PSIPHON_MESSAGE, m_sMessage, m_sIP, m_sURI, m_nUserID, m_nLastAccess, m_nWritten, m_nWriteTime, m_nRead, m_nReadTime);
	if(nLen <= 0 || nLen >= nSize)
	{
		delete pOut;
		pOut = NULL;
	}
}

bool CPsiphonMessage::Parse(const char* sIn)
{
	if(!sIn || !*sIn)
		return false;

	if(strncmp(sIn, PSIPHON_MESSAGE, min(strlen(sIn), strlen(PSIPHON_MESSAGE))) != 0)
		return false;

	char* _sIn = strdup(sIn);

	char seps[] = _T("\t");
	char* token = strtok(_sIn, seps);

	token = strtok(NULL, seps); 
	if(token == NULL)
		return false;
	SetMessage(token);

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetIP(token);

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetURI(token);

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetUserID(atoi(token));

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetLastAccess(atoi(token));

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetWritten(atoi(token));

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetWriteTime(atoi(token));

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetRead(atoi(token));

	token = strtok(NULL, seps);
	if(token == NULL)
		return false;
	SetReadTime(atoi(token));

	delete _sIn;
	return true;
}

char* CPsiphonMessage::GetMessage()
{
	return m_sMessage;
}

char* CPsiphonMessage::GetIP()
{
	return m_sIP;
}

char* CPsiphonMessage::GetURI()
{
	return m_sURI;
}

int CPsiphonMessage::GetUserID()
{
	return m_nUserID;
}

int CPsiphonMessage::GetLastAccess()
{
	return m_nLastAccess;
}

int CPsiphonMessage::GetWritten()
{
	return m_nWritten;
}

int CPsiphonMessage::GetRead()
{
	return m_nRead;
}

int CPsiphonMessage::GetWriteTime()
{
	return m_nWriteTime;
}

int CPsiphonMessage::GetReadTime()
{
	return m_nReadTime;
}

void CPsiphonMessage::SetMessage(const char* _sMessage)
{
	delete 	m_sMessage;
	m_sMessage = strdup(_sMessage);
}

void CPsiphonMessage::SetIP(const char* _sIP)
{
	delete 	m_sIP;
	m_sIP = strdup(_sIP);
}

void CPsiphonMessage::SetURI(const char* _sURI)
{
	delete 	m_sURI;

	if(_sURI && *_sURI)
		m_sURI = strdup(_sURI);
	else
		m_sURI = strdup(_T("(no URI)"));
}

void CPsiphonMessage::SetUserID(int _nUserID)
{
	m_nUserID = _nUserID;
}

void CPsiphonMessage::SetLastAccess(int _nLastAccess)
{
	m_nLastAccess = _nLastAccess;
}

void CPsiphonMessage::SetWritten(int _nWritten)
{
	m_nWritten = _nWritten;
}

void CPsiphonMessage::SetRead(int _nRead)
{
	m_nRead = _nRead;
}

void CPsiphonMessage::SetWriteTime(int _nWriteTime)
{
	m_nWriteTime = _nWriteTime;
}

void CPsiphonMessage::SetReadTime(int _nReadTime)
{
	m_nReadTime = _nReadTime;
}
