////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#pragma once

class CParsedUrl 
{
private:
    char* m_sUrl;                //Original URL      
    char* m_sProto;				// Protocol 
	char* m_sHost;				// Url host name 
	int	  m_nPort;				// Port number
	char* m_sPath;				// Uri path name 
	char* m_sQuery;				// Query string
	bool  m_bSSL;				// is SSL/secure
	int	  m_nParseError;		// Parsing error code

public:
	enum eParseError 
	{
		BAD_SCHEME = 0,		//unsupported ot missing scheme
		BAD_HOST,			//missing or invalid hostname
		BAD_PORT_NUMBER		//invalid port number
	};

	CParsedUrl();
	CParsedUrl(char* url);
	~CParsedUrl();

public:
	void Init();
    int Parse(const char* _sUrl);

public:
	const char* GetURL() const;		             	
	void SetURL(const char* pURL);

	const char* GetProtocol() const;	             	
	void SetProtocol(const char* sProto);

	int  GetPort() const;		
	void SetPort(int nPort);

	const char* GetPath() const;		           	
	void SetPath(const char* sPath, int nLength = 0);

	bool GetSSL() const;		            	
	void SetSSL(bool bSSL);
	
	const char* GetHost() const;		            	
	void  SetHost(const char* sHost, int nLength);

	const char* GetQuery() const;		            	
	void SetQuery(const char* sQuery, int nLength);
	
	int	 GetError() const;		              	

protected:
	void SetError(eParseError err);
	
	const char* ExtractProtocol(const char* pURL);
	const char*	SkipToHost (const char *p);
};
