////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "sessions.h"
#include "CppSQLite3.h"
#include "appWeb/appWeb.h"
#include "strfunc.h"

#define _T(X) X

char* CDBSessions::m_sDB = _T("sessions.db");

int CDBSessions::CreateDBSession(int nUserID, const char* sSessionVal)
{
	if(nUserID <= 0)
		return -1; 

	if(sSessionVal == NULL || strlen(sSessionVal) > 64)
		return -1; 

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("sessions")))
		{
			db.execDML(_T("create table sessions(nIndex integer primary key, nUserID int, sSessionVal char(64));"));
			db.execDML(_T("create unique index idx_session on sessions(sSessionVal ASC);"));
		}

		CppSQLite3Statement update_stmt = db.compileStatement(_T("update sessions set sSessionVal=? where nUserID=?;"));
		update_stmt.bind(1, sSessionVal);
		update_stmt.bind(2, nUserID);
		int nRows = update_stmt.execDML();

		if(nRows < 1)
		{
			CppSQLite3Statement insert_stmt = db.compileStatement(_T("insert into sessions(sSessionVal, nUserID) values(?, ?);"));
			insert_stmt.bind(1, sSessionVal);
			insert_stmt.bind(2, nUserID);
			nRows = insert_stmt.execDML();
		}

		return nRows;
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBSessions::CreateDBSession"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}
	
	return -1; 
}

int CDBSessions::FindDBSession(const char* sSessionVal, int& nUserID)
{
	if(sSessionVal == NULL || strlen(sSessionVal) > 64)
		return -1; 

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("sessions")))
			return 0;

		CppSQLite3Statement stmt = db.compileStatement(_T("select nUserID from sessions where sSessionVal=?;"));

		stmt.bind(1, sSessionVal);

		CppSQLite3Query q = stmt.execQuery();

		if(!q.eof())
		{
			nUserID = (q.getIntField(0));
		}

		return 0;
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBSessions::FindDBSession"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return -1;
}

int CDBSessions::DeleteDBSession(int nUserID)
{
	if(nUserID <= 0)
		return -1; 

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("sessions")))
			return -1;

		CppSQLite3Statement stmt = db.compileStatement(_T("DELETE FROM sessions WHERE nUserID = ?;"));

		stmt.bind(1, nUserID);

		stmt.execDML();
		return 0;
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBSessions::DeleteDBSession"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return -1; 
}

