////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include	"myclient.h"

void readEventWrapper(void* data, MprSocket* sp, int mask, int isPool);

class CClients
{
	MprList		myClients;
	MprMutex* 	m_mutex;				// Mutli-thread sync

public:
	CClients()
	{
		m_mutex = new MprMutex();
	}

	~CClients()
	{
		delete m_mutex;
	}

	void Add(MaClientEx* pClient)
	{
		CLocker lock(m_mutex);
		myClients.insert(pClient);
	}
	void Remove(MaClientEx* pClient)
	{
		CLocker lock(m_mutex);
		myClients.remove(pClient);
	}
	bool IsStillListening(MaClientEx* pClient)
	{
		CLocker lock(m_mutex);
		MaClientEx* cp = (MaClientEx*)myClients.getFirst();

		while(cp)
		{
			if(cp == (MaClientEx *)pClient)
				return true;

			cp = (MaClientEx *)myClients.getNext(cp);
		}

		return false;
	}
};

CClients g_ListneningClients;

MaClientEx::MaClientEx()
{
	m_authCnonce = 0;
	m_authNc = 0;
	m_serverAlgorithm = 0;
	m_serverDomain = 0;
	m_serverNonce = 0;
	m_serverOpaque = 0;
	m_serverRealm = 0;
	m_serverQop = 0;
	m_serverStale = 0;
	m_serverAuthType = 0;

	m_contentLength = 0;
	m_contentRemaining = -1;

	m_currentHost = 0;
	m_currentPort = -1;

	m_defaultHost = 0;
	m_defaultPort = -1;

	m_errorMsg = 0;
	m_flags = 0;
	m_fd = -1;

	m_inBuf = new MprBuf(MPR_HTTP_CLIENT_BUFSIZE + 1, -1);
	m_headerValues = 0;

	m_method = 0;
	m_outBuf = new MprBuf(MPR_HTTP_CLIENT_BUFSIZE, MPR_HTTP_CLIENT_BUFSIZE);
	m_password = 0;
	m_proxyHost = 0;
	m_proxyPort = -1;
	m_realm = 0;
	m_responseCode = -1;
	m_responseProto = 0;
	m_responseContent = new MprBuf(MPR_HTTP_CLIENT_BUFSIZE, -1);
	m_responseHeader = new MprBuf(MPR_HTTP_CLIENT_BUFSIZE, MPR_HTTP_MAX_HEADER);
	m_responseText = 0;
	m_sock = 0;
	m_state = MPR_HTTP_CLIENT_START;
	m_user = 0;
	m_userFlags = MPR_HTTP_KEEP_ALIVE;
	m_userHeaders = 0;

	m_GotStuffCond = new MprCond();

	m_mutex = new MprMutex();
}

MaClientEx::~MaClientEx()
{
	lock();

	SocketShutdown();

	delete m_headerValues;
	delete m_inBuf;
	delete m_outBuf;
	delete m_responseContent;
	delete m_responseHeader;

	mprFree(m_authCnonce);
	mprFree(m_method);
	mprFree(m_serverAlgorithm);
	mprFree(m_serverDomain);
	mprFree(m_serverNonce);
	mprFree(m_serverOpaque);
	mprFree(m_serverRealm);
	mprFree(m_serverQop);
	mprFree(m_serverStale);
	mprFree(m_serverAuthType);
	mprFree(m_errorMsg);
	mprFree(m_defaultHost);
	mprFree(m_proxyHost);
	mprFree(m_realm);
	mprFree(m_responseProto);
	mprFree(m_responseText);
	mprFree(m_password);
	mprFree(m_user);
	mprFree(m_userHeaders);

	delete m_GotStuffCond;

	unlock();
	delete m_mutex;
}

void MaClientEx::reset()
{
	m_flags = 0;

	if(m_userFlags & MPR_HTTP_KEEP_ALIVE)
	{
		m_flags |= MPR_HTTP_KEEP_ALIVE;
	}

	m_state = MPR_HTTP_CLIENT_START;
	m_contentLength = 0;
	m_contentRemaining = -1;
	m_responseCode = -1;

	if(m_method)
	{
		mprFree(m_method);
		m_method = 0;
	}
	if(m_errorMsg)
	{
		mprFree(m_errorMsg);
		m_errorMsg = 0;
	}

	delete m_headerValues;
	m_headerValues = new MprHashTable(31);

	m_outBuf->flush();
	m_responseContent->flush();
	m_responseHeader->flush();

	if(m_responseProto)
	{
		mprFree(m_responseProto);
		m_responseProto = 0;
	}

	if(m_responseText)
	{
		mprFree(m_responseText);
		m_responseText = 0;
	}

	SocketShutdown();
}

int MaClientEx::sendRequest(char* host, int port, MprBuf* hdrBuf, char* postData, int postLen)
{
	reset();

	int rc = SocketInit(host, port);
	if(rc < 0)
		return rc;

	//
	//	Remove this flush when pipelining is supported
	//
	m_inBuf->flush();
	m_fd = SocketGetFd();

	//
	//	Flush to the socket with any post data. Writes can fail because the
	//	server prematurely closes a keep-alive connection.
	//
	int len = hdrBuf->getLength();

	if((rc = SocketWrite(hdrBuf->getStart(), len)) != len)
	{
		m_flags |= MPR_HTTP_TERMINATED;
		SocketShutdown();
		return rc;
	}

	hdrBuf->addNull();

	if(postData && *postData)
	{
		if((rc = SocketWrite(postData, postLen)) != postLen)
		{
			m_flags |= MPR_HTTP_TERMINATED;
			SocketShutdown();
			return rc;
		}
	}

	g_ListneningClients.Add(this);

	SocketSetCallBack(readEventWrapper, (void*) this, 0, MPR_READABLE);

	//	If no callback, then we must block
	rc = 0;

	while(m_state != MPR_HTTP_CLIENT_DONE && mprGetMpr()->isRunningEventsThread())
	{
		int nTm = m_GotStuffCond->waitForCond(6 * 1000);

		if(nTm == -1) //timeout
		{
			rc = MPR_ERR_TIMEOUT;
			break;
		}

		char* cont_type = getHeaderVar("content-type");

		if(cont_type)
		{
			if(!IsAllowedContentType(cont_type))
			{
				rc = MPR_ERR_BAD_TYPE;
				break;
			}
		}
	}

	g_ListneningClients.Remove(this);
	SocketShutdown();

	return rc;
}

void readEventWrapper(void* data, MprSocket*, int, int)
{
	MaClientEx* cp = (MaClientEx*)data;

	if(g_ListneningClients.IsStillListening(cp) == false)
		return;

	cp->lock();

	int moreData = 0;

	do
	{
		moreData = cp->readEvent();

		if(cp->getState() == MPR_HTTP_CLIENT_DONE)
		{
			cp->signalGotStuff();
			break;
		}

		cp->signalGotStuff();
	}
	while(moreData > 0);

	cp->unlock();
}

int MaClientEx::readEvent()
{
	int len = m_inBuf->getLinearSpace();

	if(m_contentRemaining > 0 && m_state == MPR_HTTP_CLIENT_CONTENT)
	{
		len = (m_contentRemaining > len) ? len : m_contentRemaining;
	}

	int nbytes = SocketRead(m_inBuf->getEnd(), len);

	if(nbytes < 0)
	{
		// Disconnect
		if(m_state != MPR_HTTP_CLIENT_DONE && (m_state != MPR_HTTP_CLIENT_CONTENT || m_contentRemaining > 0))
		{
			m_flags |= MPR_HTTP_TERMINATED;
		}

		finishRequest(1);
		return -1;
	}
	else if(nbytes == 0)
	{
		if(SocketGetEOF())
		{
			// EOF
			if(m_state != MPR_HTTP_CLIENT_DONE && (m_state != MPR_HTTP_CLIENT_CONTENT || m_contentRemaining > 0))
			{
				m_flags |= MPR_HTTP_TERMINATED;
			}

			finishRequest(1);
			return -1;
		}
		else
		{
			// No data available
			//
			//	This happens because we call readEvent multiple times for one
			//	select event so we can read all the data possible
			//
			return 0;
		}
	}
	else
	{
		// Data available
		m_inBuf->adjustEnd(nbytes);
		m_inBuf->addNull();
		processResponseData();

		//	Request may or may not have completed yet
		return 1;
	}
}

void MaClientEx::signalGotStuff()
{
	m_GotStuffCond->signalCond();
}

void MaClientEx::finishRequest(bool closeSocket)
{
	m_state = MPR_HTTP_CLIENT_DONE;
	m_responseContent->addNull();
	m_responseHeader->addNull();

	if(!closeSocket)
	{
		SocketReset();
	}
	else
	{
		SocketShutdown();
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////

int MaClientEx::SocketGetFd()
{
	if(m_sock == NULL)
		return -1;

	CSocketLocker lock(m_sock);
	return m_sock->getFd();
}

bool MaClientEx::SocketGetEOF()
{
	if(m_sock == NULL)
		return true;

	CSocketLocker lock(m_sock);

	return m_sock->getEof();
}

void MaClientEx::SocketReset()
{
	if(m_sock == NULL)
		return;

	CSocketLocker lock(m_sock);

	m_sock->setCallback(readEventWrapper, (void*) this, 0, 0);
}

int MaClientEx::SocketInit(char* host, int port)
{
	if(!SocketCreate())
		return -1;

	int rc = SocketOpenClient(host, port, MPR_SOCKET_NODELAY);

	if(rc < 0)
	{
		SocketShutdown();
		return rc;
	}

	SocketSetBufSize(-1, MPR_HTTP_CLIENT_BUFSIZE);

	return 0;
}

int MaClientEx::SocketSetBufSize(int sendSize, int recvSize)
{
	if(m_sock == NULL)
		return -1;

	CSocketLocker lock(m_sock);

	return m_sock->setBufSize(sendSize, recvSize);
}

bool MaClientEx::SocketCreate()
{
	SocketShutdown();
	m_sock = new MprSocket();
	return m_sock != NULL;
}

int MaClientEx::SocketOpenClient(char* hostName, int portNum, int flags)
{
	if(m_sock == NULL)
		return -1;

	CSocketLocker lock(m_sock);
	return m_sock->openClient(hostName, portNum, flags); //MPR_SOCKET_BLOCK
}

void MaClientEx::SocketShutdown()
{
	CLocker lock(m_mutex);

	if(m_sock == NULL)
		return;

	m_sock->lock();

	m_sock->setCallback(/*readEventWrapper*/NULL, (void*) this, 0, 0);
	m_sock->close(MPR_SHUTDOWN_BOTH);

	m_sock->unlock();

	m_sock->dispose();
	m_sock = 0;
}

void MaClientEx::SocketSetCallBack(MprSocketIoProc fn, void* data, void* data2, int mask, int pri)
{
	if(m_sock == NULL)
		return;

	CSocketLocker lock(m_sock);

	m_sock->setCallback(fn, data, data2, mask, pri);
}

int MaClientEx::SocketWrite(char* buf, int len)
{
	if(m_sock == NULL)
		return -1;

	CSocketLocker lock(m_sock);

	return m_sock->write(buf, len);
}

int MaClientEx::SocketRead(char* buf, int len)
{
	if(m_sock == NULL)
		return -1;

	CSocketLocker lock(m_sock);
	return m_sock->read(buf, len);
}

/////////////////////////////////////////////////////////////////////////////////////////////

int MaClientEx::processResponseData()
{
	char* cp = NULL;
	int nbytes = 0;

	char* line = NULL;

	while(m_state != MPR_HTTP_CLIENT_DONE && m_inBuf->getLength() > 0)
	{
		line = m_inBuf->getStart();

		if(m_state != MPR_HTTP_CLIENT_CONTENT)
		{
			if((cp = strchr(line, '\n')) == 0)
			{
				//	Wait for more data
				return 0;
			}

			*cp = '\0';

			if(cp[-1] == '\r')
			{
				nbytes = cp - line;
				*--cp = '\0';
			}
			else
			{
				nbytes = cp - line;
			}

			m_inBuf->adjustStart(nbytes + 1);
		}
		else
		{
			if(m_contentLength <= 0)
			{
				nbytes = m_inBuf->getLength();
			}
			else
			{
				nbytes = min(m_contentRemaining, m_inBuf->getLength());
			}

			m_inBuf->adjustStart(nbytes);
		}

		switch(m_state)
		{
		case MPR_HTTP_CLIENT_START:
			if(line[0] == '\0')
			{
				return 0;
			}

			m_responseHeader->put(line);
			m_responseHeader->put('\n');

			if(parseFirst(line) < 0)
			{
				return MPR_ERR_BAD_STATE;
			}

			m_state = MPR_HTTP_CLIENT_HEADER;
			break;

		case MPR_HTTP_CLIENT_HEADER:
			if(nbytes > 1)
			{
				m_responseHeader->put(line);
				m_responseHeader->put('\n');

				if(parseHeader(line) < 0)
				{
					return MPR_ERR_BAD_STATE;
				}
			}
			else
			{
				m_state = MPR_HTTP_CLIENT_CONTENT;
				//
				//	This means there was an explicit zero content length
				//
				if(m_contentRemaining == 0)
				{
					finishRequest(0);
				}
				else if(mprStrCmpAnyCase(m_method, "HEAD") == 0)
				{
					finishRequest(0);
				}
			}
			break;

		case MPR_HTTP_CLIENT_CONTENT:
			m_responseContent->put((uchar *)line, nbytes);
			m_responseContent->addNull();

			if(m_contentRemaining > 0 || nbytes <= 0)
			{
				m_contentRemaining -= nbytes;

				if(m_contentRemaining <= 0)
					finishRequest(0);
			}
			break;

		default:
			m_responseCode = MPR_HTTP_CLIENT_ERROR;
			finishRequest(1);

			return MPR_ERR_BAD_STATE;
		}
	}

	return 0;
}

int MaClientEx::parseFirst(char* line)
{
	char* code, * tok;

	mprAssert(line && *line);

	m_responseProto = mprStrTok(line, " \t", &tok);

	if(m_responseProto == 0 || m_responseProto[0] == '\0')
	{
		m_responseCode = MPR_HTTP_CLIENT_ERROR;
		finishRequest(1);
		return MPR_ERR_BAD_STATE;
	}

	m_responseProto = mprStrdup(m_responseProto);

	if(strncmp(m_responseProto, "HTTP/1.", 7) != 0)
	{
		m_responseCode = MPR_HTTP_CLIENT_ERROR;
		finishRequest(1);
		return MPR_ERR_BAD_STATE;
	}

	code = mprStrTok(0, " \t\r\n", &tok);

	if(code == 0 || *code == '\0')
	{
		m_responseCode = MPR_HTTP_CLIENT_ERROR;
		finishRequest(1);
		return MPR_ERR_BAD_STATE;
	}

	m_responseCode = atoi(code);

	m_responseText = mprStrTok(0, "\r\n", &tok);

	if(m_responseText && *m_responseText)
	{
		m_responseText = mprStrdup(m_responseText);
	}

	return 0;
}

int MaClientEx::parseHeader(char* line)
{
	char* key, * value, * tok, * tp;

	mprAssert(line && *line);

	if((key = mprStrTok(line, ": \t\n", &tok)) == 0)
	{
		m_responseCode = MPR_HTTP_CLIENT_ERROR;
		finishRequest(1);
		return MPR_ERR_BAD_STATE;
	}
	if((value = mprStrTok(0, "\n", &tok)) == 0)
	{
		value = "";
	}
	while(isspace(*value))
	{
		value++;
	}

	//	Upper to be consistent with Request?
	mprStrLower(key);
	m_headerValues->insert(new MprStringHashEntry(key, value));

	if(strcmp("www-authenticate", key) == 0)
	{
		tp = value;
		while(*value && !isspace(*value))
		{
			value++;
		}

		*value++ = '\0';
		mprStrLower(tp);
		mprFree(m_serverAuthType);
		m_serverAuthType = mprStrdup(tp);

		if(parseAuthenticate(value) < 0)
		{
			m_responseCode = MPR_HTTP_CLIENT_ERROR;
			finishRequest(1);
			return MPR_ERR_BAD_STATE;
		}
	}
	else if(strcmp("content-length", key) == 0)
	{
		m_contentLength = atoi(value);

		if(mprStrCmpAnyCase(m_method, "HEAD") != 0)
		{
			m_contentRemaining = atoi(value);
		}
	}
	else if(strcmp("connection", key) == 0)
	{
		mprStrLower(value);
		if(strcmp(value, "close") == 0)
		{
			m_flags &= ~MPR_HTTP_KEEP_ALIVE;

#if BLD_FEATURE_KEEP_ALIVE
		}
		else if(strcmp(value, "keep-alive") == 0)
		{
			if(m_userFlags & MPR_HTTP_KEEP_ALIVE)
			{
				m_flags |= MPR_HTTP_KEEP_ALIVE;
			}
#endif
		}
	}
	else if(strcmp("transfer-encoding", key) == 0)
	{
		mprStrLower(value);
		if(strcmp(value, "chunked") == 0)
		{
			m_flags |= MPR_HTTP_INPUT_CHUNKED;
		}
	}
	return 0;
}

int MaClientEx::parseAuthenticate(char* authDetails)
{
	char* value, * tok, * key, * dp, * sp;
	int seenComma;

	key = authDetails;

	while(*key)
	{
		while(*key && isspace(*key))
		{
			key++;
		}
		tok = key;

		while(*tok && !isspace(*tok) && *tok != ',' && *tok != '=')
		{
			tok++;
		}

		*tok++ = '\0';

		while(isspace(*tok))
		{
			tok++;
		}

		seenComma = 0;

		if(*tok == '\"')
		{
			value = ++tok;
			while(*tok != '\"' && *tok != '\0')
			{
				tok++;
			}
		}
		else
		{
			value = tok;
			while(*tok != ',' && *tok != '\0')
			{
				tok++;
			}
			seenComma++;
		}
		*tok++ = '\0';

		//
		//	Handle back-quoting
		//
		if(strchr(value, '\\'))
		{
			for(dp = sp = value;*sp;sp++)
			{
				if(*sp == '\\')
				{
					sp++;
				}
				*dp++ = *sp++;
			}
			*dp = '\0';
		}

		//
		//	algorithm, domain, nonce, oqaque, realm, qop, stale
		//	
		switch(tolower(*key))
		{
		case 'a':
			if(mprStrCmpAnyCase(key, "algorithm") == 0)
			{
				mprFree(m_serverAlgorithm);
				m_serverAlgorithm = mprStrdup(value);
				break;
			}
			break;

		case 'd':
			if(mprStrCmpAnyCase(key, "domain") == 0)
			{
				mprFree(m_serverDomain);
				m_serverDomain = mprStrdup(value);
				break;
			}
			break;

		case 'n':
			if(mprStrCmpAnyCase(key, "nonce") == 0)
			{
				mprFree(m_serverNonce);
				m_serverNonce = mprStrdup(value);
				m_authNc = 0;
			}
			break;

		case 'o':
			if(mprStrCmpAnyCase(key, "opaque") == 0)
			{
				mprFree(m_serverOpaque);
				m_serverOpaque = mprStrdup(value);
			}
			break;

		case 'q':
			if(mprStrCmpAnyCase(key, "qop") == 0)
			{
				mprFree(m_serverQop);
				m_serverQop = mprStrdup(value);
			}
			break;

		case 'r':
			if(mprStrCmpAnyCase(key, "realm") == 0)
			{
				mprFree(m_serverRealm);
				m_serverRealm = mprStrdup(value);
			}
			break;

		case 's':
			if(mprStrCmpAnyCase(key, "stale") == 0)
			{
				mprFree(m_serverStale);
				m_serverStale = mprStrdup(value);
				break;
			}

		default:
			//	For upward compatibility --  ignore keywords we don't understand
			;
		}

		key = tok;
		if(!seenComma)
		{
			while(*key && *key != ',')
			{
				key++;
			}
			if(*key)
			{
				key++;
			}
		}
	}
	if(strcmp(m_serverAuthType, "basic") == 0)
	{
		if(m_serverRealm == 0)
		{
			return MPR_ERR_BAD_ARGS;
		}
		else
		{
			return 0;
		}
	}
	if(m_serverRealm == 0 || m_serverNonce == 0)
	{
		return MPR_ERR_BAD_ARGS;
	}

	if(m_serverQop)
	{
		//	FUTURE -- add checking for auth-int here
		if(m_serverDomain == 0 || m_serverOpaque == 0 || m_serverAlgorithm == 0 || m_serverStale == 0)
		{
			return MPR_ERR_BAD_ARGS;
		}
	}
	return 0;
}

int MaClientEx::getResponseCode()
{
	if(m_state != MPR_HTTP_CLIENT_DONE)
	{
		return MPR_ERR_BAD_STATE;
	}

	return m_responseCode;
}

char* MaClientEx::getResponseContent(int* contentLen)
{
	mprAssert(contentLen);

	if(m_state != MPR_HTTP_CLIENT_DONE)
	{
		return 0;
	}

	if(contentLen)
	{
		*contentLen = m_responseContent->getLength();
	}

	if(m_responseContent->getStart() == 0)
		return "";

	return m_responseContent->getStart();
}

char* MaClientEx::getResponseHeader()
{
	if(m_state != MPR_HTTP_CLIENT_DONE)
	{
		return 0;
	}

	if(m_responseHeader->getStart() == 0)
	{
		return "";
	} 

	return m_responseHeader->getStart();
}

char* MaClientEx::getHeaderVar(char* key)
{
	MprStringHashEntry* hp;

	hp = (MprStringHashEntry *)m_headerValues->lookup(key);
	if(hp)
		return hp->getValue();
	return NULL;
}

void MaClientEx::setProxy(char* host, int port)
{
	if(m_proxyHost)
	{
		mprFree(m_proxyHost);
	}
	m_proxyHost = mprStrdup(host);
	m_proxyPort = port;
}

void MaClientEx::getParsedUrl(MaUrl** urlp)
{
	mprAssert(urlp);

	if(urlp)
	{
		*urlp = &m_url;
	}
}

int MaClientEx::getState()
{
	return m_state;
}

int MaClientEx::getFlags()
{
	return m_flags;
}

int MaClientEx::getPort()
{
	return m_defaultPort;
}

char* MaClientEx::getHost()
{
	return m_defaultHost;
}

void MaClientEx::setPort(int num)
{
	m_defaultPort = num;
}

void MaClientEx::setHost(char* host)
{
	if(m_defaultHost)
	{
		mprFree(m_defaultHost);
	}
	m_defaultHost = mprStrdup(host);
}

void MaClientEx::setAuth(char* _realm, char* _username, char* _password)
{
	mprFree(m_user);
	mprFree(m_realm);
	mprFree(m_password);

	m_realm = mprStrdup(_realm);
	m_user = mprStrdup(_username);
	m_password = mprStrdup(_password);
}

void MaClientEx::resetAuth()
{
	mprFree(m_serverAuthType);
	m_serverAuthType = 0;
}

void MaClientEx::setUserHeaders(char* headers)
{
	mprFree(m_userHeaders);
	m_userHeaders = mprStrdup(headers);
}

void MaClientEx::allowContentType(std::string allowed_content)
{
	m_allowedVector.push_back(allowed_content);
}

bool MaClientEx::IsAllowedContentType(char* cont_type)
{
	const char* allowed = NULL;

	if(m_allowedVector.empty())
		return false;

	for(int i = 0;i < (int)m_allowedVector.size();i++)
	{
		std::string stdStr = m_allowedVector.at(i);
		allowed = stdStr.c_str();

		if(allowed && *allowed)
		{
			if(strncmp(cont_type, allowed, min(strlen(cont_type), strlen(allowed))) == 0)
				return true;
		}
	}

	return false;
}
