////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#pragma once

#define MAX_REDIRECTS 5
#define MAX_LANG_SIZE 20

///////define request URIs
#define URL_SERVEFILE			"/html/images/"
#define URL_LOGIN				"/login/"
#define URL_LOGIN_NOSLASH		"/login"
#define URL_CHANGEPWD			"/change-pwd/"
#define URL_CHANGEPWD_NOSLASH	"/change-pwd"
#define URL_GETBOOKMARKS		"/bookmarks/"
#define URL_ADDBOOKMARK			"/add-bookmark/"
#define URL_DELETEBOOKMARK		"/delete-bookmark/"
#define URL_FETCH				"/fetch/"
#define URL_FETCH_NOSLASH		"/fetch"
#define URL_FETCHI				"/fetch-i/"
#define URL_FETCHI_NOSLASH		"/fetch-i"
#define URL_TOPBARFRAME			"/topbar-frame/"
#define URL_SETLANG				"/set-lang/"

///////define request URI parsed values
#define SERVEFILE			1
#define LOGIN				2
#define LOGIN_REDIRECT		3
#define CHANGEPWD			4
#define CHANGEPWD_REDIRECT	5
#define GETBOOKMARKS		6
#define ADDBOOKMARK			7
#define DELETEBOOKMARK		8
#define FETCH				9
#define FETCH_REDIRECT		10
#define FETCHI				11
#define FETCHI_REDIRECT		12
#define TOPBARFRAME			13
#define SETLANG				14
#define INDEX				15
#define NOTVALID			16
#define MAINPAGE			17

///////define filtering flags
#define NO_FILTER			1
#define HTML_FILTER			2
#define CSS_FILTER			3

#include	"appWeb/appWeb.h"
#include	"myclient.h"
#include	"User.h"
#include	"psiphonUtils.h"

/////////////////////////// Extern Definitions /////////////////////////

class PsiphonHandler;
class PsiphonHandlerService;
class CStringReplaceData;

extern "C" {
extern int mprPsiphonHandlerInit(void* handle);
};

////////////////////////////////////////////////////////////////////////
///////////////////////// PsiphonHandlerModule //////////////////////////
////////////////////////////////////////////////////////////////////////

class PsiphonHandlerModule : public MaModule
{
private:
	PsiphonHandlerService* 	simpleService;

public:
							PsiphonHandlerModule(void* handle);
							~PsiphonHandlerModule();
};

////////////////////////////////////////////////////////////////////////
///////////////////////////// PsiphonHandler ////////////////////////////
////////////////////////////////////////////////////////////////////////

class PsiphonHandlerService : public MaHandlerService
{
public:
				PsiphonHandlerService();
				~PsiphonHandlerService();

public:
	MaHandler*	newHandler(MaServer* server, MaHost* host, char* ext);
	int			setup();
};

class PsiphonHandler : public MaHandler
{
private:
	MprBuf* 			m_pPostBuf;
	CDBUser				m_currentUser;
	char* 				m_pFetchURL;
	char* 				m_pRequestURL;

public:
	MaRequest* 			m_pRq;
	bool				m_bTopBar;
	int					m_bServerShowImages;
	int					m_bClientShowImages;
	char* 				m_pNodeName;

public:
						PsiphonHandler(char* extensions);
	virtual				~PsiphonHandler();

public:
	virtual	MaHandler*	cloneHandler();
	virtual int			matchRequest(MaRequest* rq, char* uri, int uriLen);
	virtual void		postData(MaRequest* rq, char* buf, int buflen);
	virtual int			run(MaRequest* rq);
	virtual int			setup(MaRequest* rq);

public:
	int					Write(const char* _pBuf, int nSize = -1);

protected:
	void				Init();
	int					MatchRequest(char* uri, int uriLen);

	void				PostData(char* buf, int buflen);
	void				SetSessionData();
	int					IsLogged();

	void				Login();
	bool				TryToLogin(char*& filename, char*& user);
	void				ShowLoginPage(char* form_error, char* user);

	void				ShowIndexPage();
	bool				IsIndexPage();
	void				SendTopBar();
	void				RedirectSpecial(char* sURL);

	int					CheckExpiredPassword();
	void				ResetPassword();
	int					ServeRequest();
	void				ServeFile();
	int					InsertNavigationBar(char* request_uri); 

	void				LogRequest();

	int					ValidateRequest(char*& http_method);
	int					ValidateRequestURI();
	int					ValidateRequestProtocol(const char* url);
	int					RedirectRequestURI(const MaUrl& maUrl, char*& request_uri);
	int					PrepareHeader(MprBuf& bufHeader, char* http_method, char* request_uri, const MaUrl& maUrl);
	int					GetCookies(MprBuf& bufHeader, char* request_uri);
	int					StoreCookies(char* response_header, char* request_uri);
	int					ExamineResponseCode(int nCode, MaClientEx* pClient, char* request_uri);
	int					WriteResponse(char* cont_type, char* content, int contentLen, char* request_uri);
	int					GetClientResponseContent(MaClientEx* pClient, char*& cont_type, char*& content, int& contentLen);
	void				SendBookmarks();
	void				AddBookmark();
	void				DeleteBookmark();
	int					GetLanguagesStr(CLanguageData lang[], char** pLanguages);
	bool				IsExternalHost(char* pUri);
	void				ShowMainPage();
	void				SetLanguageCookie();
	int					Fetch();
	int					ReplaceWrite(char* buf, std::vector <CStringReplaceData> replaceArr);

	void				FilterWrite(int cont_type, char* content, int contentLen, char* request_uri);
	int					RequestError(int nCode);
	void				SetClientShowImages();

	const char*			GetCleanURI(const char* pURI);

};

class CStringReplaceData
{
public:
	const char* m_varname;
	const char* m_replacement;

	CStringReplaceData(){}
	CStringReplaceData(const char* varname, const char* replacement) : m_varname(varname), m_replacement(replacement)
	{
	}

	~CStringReplaceData()
	{
	}
};
