////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "psiphonUrl.h"
#include "appWeb/appWeb.h"

/*
const scheme_data CPsiphonURL::m_supported_schemes[]=
{
	scheme_data("http", "http://", 80, 1),
	scheme_data("https", "https://", 443, 1),
	scheme_data("ftp", "ftp://", 21, 0),
	scheme_data(0, 0, 0, 0)
};
*/
CPsiphonURL::CPsiphonURL(char* _url, char* _base)
{
	m_url = _url;
	m_base = _base;
}

CPsiphonURL::~CPsiphonURL(void)
{
}

int CPsiphonURL::SchemeChar(const char* ch)
{
	return isalnum(*ch) || (*ch) == '-' || (*ch) == '+';
}

int CPsiphonURL::URLHasScheme(const char* p)
{
	/* The first char must be a scheme char. */
	if(!*p || !SchemeChar(p))
		return 0;

	++p;

	/* Followed by 0 or more scheme chars. */
	while(*p && SchemeChar(p))
		++p;

	/* Terminated by ':'. */
	return *p == ':';
}

/* proxify all links, decide if allowed scheme later
int CPsiphonURL::IsAllowedScheme(const char* str)
{
	for (int i = 0; m_supported_schemes[i].m_leading_string; i++)
	{
		if (0 == strnicmp (str, m_supported_schemes[i].m_leading_string,
			strlen (m_supported_schemes[i].m_leading_string)))
		{
			return (m_supported_schemes[i].m_enabled);
		}
	}

	return 0;
}

*/
int CPsiphonURL::TransformURL(char*& transformed_url)
{
	if(URLHasScheme(m_url))
	{
		transformed_url = strdup(m_url); /*else*/
	}
	else
	{
		transformed_url = MergeURL();
	}
	return 1;
}

int CPsiphonURL::URLPathLength(const char* url)
{
	const char* q = strpbrk(url, "?;#");

	if(!q)
		q = strchr(url, '\0');	

	return q - url;
}

char* CPsiphonURL::MergeURL()
{
	int linklength;
	const char* end;
	char* merge;

	if(URLHasScheme(m_url))
		return strdup(m_url);

	/* We may not examine BASE past END. */
	end = m_base + URLPathLength(m_base);
	linklength = strlen(m_url);

	if(!*m_url)
	{
		/* Empty LINK points back to BASE, query string and all. */
		return strdup(m_base);
	}
	else if(*m_url == '?')
	{
		/* LINK points to the same location, but changes the query
								string.  Examples: */
		/* uri_merge("path",		 "?new") -> "path?new"     */
		/* uri_merge("path?foo",	 "?new") -> "path?new"     */
		/* uri_merge("path?foo#bar", "?new") -> "path?new"     */
		/* uri_merge("path#foo",	 "?new") -> "path?new"     */
		int baselength = end - m_base;

		merge = (char*)malloc(baselength + linklength + 1);

		memcpy(merge, m_base, baselength);
		memcpy(merge + baselength, m_url, linklength);
		merge[baselength + linklength] = '\0';
	}
	else if(*m_url == '#')
	{
		/* uri_merge("path",		 "#new") -> "path#new"     */
		/* uri_merge("path#foo",	 "#new") -> "path#new"     */
		/* uri_merge("path?foo",	 "#new") -> "path?foo#new" */
		/* uri_merge("path?foo#bar", "#new") -> "path?foo#new" */
		int baselength;
		const char* end1 = strchr(m_base, '#');

		if(!end1)
			end1 = m_base + strlen(m_base);

		baselength = end1 - m_base;
		merge = (char*)malloc(baselength + linklength + 1);
		memcpy(merge, m_base, baselength);
		memcpy(merge + baselength, m_url, linklength);
		merge[baselength + linklength] = '\0';
	}
	else if(*m_url == '/' && *(m_url + 1) == '/')
	{
		/* LINK begins with "//" and so is a net path: we need to
											replace everything after (and including) the double slash
											with LINK. */

		/* uri_merge("foo", "//new/bar")			-> "//new/bar"  	*/
		/* uri_merge("//old/foo", "//new/bar")  	-> "//new/bar"  	*/
		/* uri_merge("http://old/foo", "//new/bar") -> "http://new/bar" */

		int span;
		const char* slash;
		const char* start_insert;

		/* Look for first slash. */
		slash = (const char*)memchr(m_base, '/', end - m_base);
		/* If found slash and it is a double slash, then replace
											from this point, else default to replacing from the
											beginning.  */
		if(slash && *(slash + 1) == '/')
			start_insert = slash;
		else
			start_insert = m_base;

		span = start_insert - m_base;
		merge = (char*)malloc(span + linklength + 1);
		if(span)
			memcpy(merge, m_base, span);
		memcpy(merge + span, m_url, linklength);
		merge[span + linklength] = '\0';
	}
	else if(*m_url == '/')
	{
		/* LINK is an absolute path: we need to replace everything
											after (and including) the FIRST slash with LINK.
											So, if BASE is "http://host/whatever/foo/bar", and LINK is
											"/qux/xyzzy", our result should be
											"http://host/qux/xyzzy".  */
		int span;

		const char* slash;
		const char* start_insert = NULL; /* for gcc to shut up. */
		const char* pos = m_base;

		int seen_slash_slash = 0;
		/* We're looking for the first slash, but want to ignore
											double slash. */

		again : slash = (const char*)memchr(pos, '/', end - pos);

		if(slash && !seen_slash_slash)
			if(*(slash + 1) == '/')
			{
				pos = slash + 2;
				seen_slash_slash = 1;

				goto again;
			}

		/* At this point, SLASH is the location of the first / after
											"//", or the first slash altogether.  START_INSERT is the
											pointer to the location where LINK will be inserted.  When
											examining the last two examples, keep in mind that LINK
											begins with '/'. */

		if(!slash && !seen_slash_slash)
		/* example: "foo" */
		/*  		 ^    */
			start_insert = m_base;
		else if(!slash && seen_slash_slash)
		/* example: "http://foo" */
		/*  				   ^ */
			start_insert = end;
		else if(slash && !seen_slash_slash)
		/* example: "foo/bar" */
		/*  		 ^  	  */
			start_insert = m_base;
		else if(slash && seen_slash_slash)
		/* example: "http://something/" */
		/*  						 ^  */
			start_insert = slash;

		span = start_insert - m_base;
		merge = (char*)malloc(span + linklength + 1);

		if(span)
			memcpy(merge, m_base, span);

		memcpy(merge + span, m_url, linklength);
		merge[span + linklength] = '\0';
	}
	else
	{
		/* LINK is a relative URL: we need to replace everything
														after last slash (possibly empty) with LINK.
														So, if BASE is "whatever/foo/bar", and LINK is "qux/xyzzy",
														our result should be "whatever/foo/qux/xyzzy".  */
		int need_explicit_slash = 0;
		int span;
		const char* start_insert = 0;
		const char* last_slash = 0;

		for(;end > m_base && *(end + 1) != '/';end--)
			last_slash = end;

		if(last_slash == 0 || !last_slash)
		{
			/* No slash found at all.  Replace what we have with LINK. */
			start_insert = m_base;
		}
		else if(last_slash && last_slash >= m_base + 2 && last_slash[-2] == ':' && last_slash[-1] == '/')
		{
			/* example: http://host"  */
			/*  					^ */
			start_insert = end + 1;
			need_explicit_slash = 1;
		}
		else
		{
			/* example: "whatever/foo/bar" */
			/*  					  ^    */
			start_insert = last_slash + 1;
		}

		span = start_insert - m_base;
		merge = (char*)malloc(span + linklength + 1);

		if(span)
			memcpy(merge, m_base, span);

		if(need_explicit_slash)
			merge[span - 1] = '/';

		memcpy(merge + span, m_url, linklength);
		merge[span + linklength] = '\0';
	}

	return merge;
}


