////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "psiphonUtils.h"
#include "psiphonHandler.h"
#include "strfunc.h"

#define DEFAULT_LANGUAGE_DIR "en"
#define HTML_DIR "html"
#define LANGUAGE_FILE "language.ini"

CPsiphonUtils::CPsiphonUtils(PsiphonHandler* _pSh)
{
	m_pSh = _pSh;
}

CPsiphonUtils::~CPsiphonUtils(void)
{
}

#ifdef _WIN32
int CPsiphonUtils::GetLanguagesData(CLanguageData lang[], int nLangSize)
{
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind = INVALID_HANDLE_VALUE;

	char* docRoot = m_pSh->m_pRq->host->getDocumentRoot();

	int nSize = strlen(docRoot) + strlen(HTML_DIR) + 4; // +4 for //* and '\0'

	char* path = new char[nSize]; 
	int nLen = _snprintf(path, nSize, "%s/%s/*", docRoot, HTML_DIR);
	if(nLen <= 0 || nLen >= nSize)
	{
		delete path;
		return 1;
	}

	hFind = FindFirstFile(path, &FindFileData);

	int count = 0;
	char* temp = NULL;
	if(hFind != INVALID_HANDLE_VALUE)
	{
		do
		{
			if(FindFileData.cFileName[0] == '.')
			{
				continue;
			}

			if(FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				temp = NULL;
				temp = FindFileData.cFileName;
				if(count < nLangSize)
				{
					CLanguageData* pLangData = &lang[count];
					pLangData->SetDir(strdup(temp));

					if(ReadLanguageFile(&pLangData) == 0)
					{
						count++;
					}
				}
			}
		}
		while(FindNextFile(hFind, &FindFileData) != 0);
	}
	else
	{
		delete path;
		return 1;
	}

	FindClose(hFind);	

	if(path)
		delete path;

	return 0;
}

#else
int CPsiphonUtils::GetLanguagesData(CLanguageData lang[], int nLangSize)
{
	char* docRoot = m_pSh->m_pRq->host->getDocumentRoot();

	int nSize = strlen(docRoot) + strlen(HTML_DIR) + 2;// +2 for / and '\0'

	char* path = new char[nSize]; 
	int nLen = _snprintf(path, nSize, "%s/%s", docRoot, HTML_DIR);
	if(nLen <= 0 || nLen >= nSize)
	{
		delete path;
		return 1;
	}

	DIR* dir = opendir((char*)path);

	if(dir == 0)
	{
		delete path;
		return 1;
	}

	int count = 0;
	char* temp = NULL;
	struct dirent* dirent = NULL;
	while((dirent = readdir(dir)) != 0)
	{
		if(dirent->d_name[0] == '.')
		{
			continue;
		}

		int nSize = strlen(path) + strlen(dirent->d_name) + 2;

		char* s_dir_name = new char[nSize];
		int nLen = _snprintf(s_dir_name, nSize, "%s/%s", path, dirent->d_name);
		if(nLen <= 0 || nLen >= nSize)
		{
			delete s_dir_name;
			return 1;
		}

		DIR* temp_dir = opendir((char*)s_dir_name);

		if(s_dir_name)
			delete s_dir_name;

		if(temp_dir != 0)
		{
			temp = NULL;
			temp = dirent->d_name; 	
			closedir(temp_dir);

			if(count < nLangSize)
			{
				CLanguageData* pLangData = &lang[count];
				pLangData->SetDir(strdup(temp));

				if(ReadLanguageFile(&pLangData) == 0)
				{
					count++;
				}
			}
		}
	}
	closedir(dir);

	return 0;
}

#endif

const char* CPsiphonUtils::GetCookieData(const char* cookieName)
{
	// 
	// Parse the cookie 
	// 
	if(!m_pSh->m_pRq->getCookie())
		return NULL; 

	// 
	// Now break up the cookies into a single cookie and search for the 
	// cookie we're looking for.  Cookies are separated by '; ' 
	// 
	char* cookie = NULL;
	char* value = NULL; 

	if((cookie = strstr(m_pSh->m_pRq->getCookie(), cookieName)) == NULL)
		return NULL; 

	// Get the value string in our cookie. 
	m_pSh->m_pRq->getCrackedCookie(cookie, 0, &value, 0); 

	return value;
} 

int CPsiphonUtils::GetHTML(const char* template_filename, char*& content)
{
	if(template_filename == NULL)
		return 1;

	const char* language_dir = GetLanguageDir();
	int lang_err = 0;

	if((lang_err = GetLanguageHTML(template_filename, content, language_dir)) != 0)
	{
		if(!strcmp(DEFAULT_LANGUAGE_DIR, language_dir)) //already tried default_dir
		{
			free((void*)language_dir);
			return lang_err;
		}
		else
		{
			free((void*)language_dir);
			return GetLanguageHTML(template_filename, content, DEFAULT_LANGUAGE_DIR);
		}
	}

	return 0;
}

int CPsiphonUtils::GetLanguageHTML(const char* template_filename, char*& content, const char* language_dir)
{
	FILE* pFile = NULL;
	char* html_dir = HTML_DIR;

	char* docRoot = m_pSh->m_pRq->host->getDocumentRoot();

	int nSize = strlen(docRoot) + strlen(html_dir) + strlen(language_dir) + strlen(template_filename) + 4; //3 slashes + '\0'

	char* template_path = new char[nSize]; 
	int nLen = _snprintf(template_path, nSize, "%s/%s/%s/%s", docRoot, html_dir, language_dir, template_filename);
	if(nLen <= 0 || nLen >= nSize)
	{
		delete template_path;
		return -1;
	}

	pFile = fopen(template_path, "rb");

	delete template_path;

	if(pFile == NULL)
		return -1;

	// obtain file size.
	fseek(pFile, 0, SEEK_END);
	long nFileLength = ftell(pFile);
	fseek(pFile, 0, SEEK_SET);

	// allocate memory to contain the whole file.
	content = new char[nFileLength + 1];

	// copy the file into the buffer.
	fread(content, 1, nFileLength, pFile);
	content[nFileLength] = '\0';

	// terminate
	fclose(pFile);

	return 0;
}

const char* CPsiphonUtils::GetLanguageDir()
{
	const char* pLangVal = GetCookieData("psi_lang");

	if(!pLangVal || !*pLangVal)
		return strdup(DEFAULT_LANGUAGE_DIR);

	return pLangVal;
}

int CPsiphonUtils::ReadLanguageFile(CLanguageData** language)
{
	FILE* pFile = NULL;
	char* html_dir = HTML_DIR;
	char* language_filename = LANGUAGE_FILE;

	char* docRoot = m_pSh->m_pRq->host->getDocumentRoot();

	int nSize = strlen(docRoot) + strlen(html_dir) + strlen((*language)->GetDir()) + strlen(language_filename) + 4; //3 slashes + '\0'

	char* template_path = new char[nSize];
	int nLen = _snprintf(template_path, nSize, "%s/%s/%s/%s", docRoot, html_dir, (*language)->GetDir(), language_filename);
	if(nLen <= 0 || nLen >= nSize)
	{
		delete template_path;
		return -1;
	}

	pFile = fopen(template_path, "rb");

	delete template_path;

	if(pFile == NULL)
		return -1;

	// obtain file size.
	fseek(pFile, 0, SEEK_END);
	long nFileLength = ftell(pFile);

	fseek(pFile, 0, SEEK_SET);

	if(!nFileLength)
		return -2;

	char* temp = new char[nFileLength + 1];

	// copy the file into the buffer.
	fread(temp, 1, nFileLength, pFile);
	temp[nFileLength] = '\0';
	// terminate
	fclose(pFile);
	(*language)->SetLanguage(temp);

	return 0;
}

bool CPsiphonUtils::IsExternalIP(const char* pIP)
{
	if(strcmp(pIP, "127.0.0.1") == 0) //localhost
		return false;

	if(strncmp(pIP, "10.", 3) == 0) //10.0.0.0  10.255.255.255
		return false;

	if(strncmp(pIP, "192.168", 7) == 0) //192.168.0.0  192.168.255.255
		return false;

	if(strncmp(pIP, "169.254", 7) == 0) //169.254.0.0  169.254.255.255
		return false;

	if(strncmp(pIP, "172.", 4) == 0)  //172.16.0.0  172.31.255.255
	{
		int nSec = 0;
		char* tempIP = strdup(pIP);

		if(!tempIP)
			return false;

		char* pSec = strtok(tempIP + 4, ".");

		if(pSec)
			nSec = atoi(pSec);

		delete tempIP;

		if(nSec >= 16 && nSec <= 31)
			return false;
	}

	return true;
}

CLanguageData::CLanguageData()
{
	m_dir = NULL;
	m_language = NULL;
}

CLanguageData::~CLanguageData()
{
	if(m_dir)
		delete m_dir;

	if(m_language)
		delete m_language;
}

char* CLanguageData::GetDir()
{
	return m_dir;
}

char* CLanguageData::GetLanguage()
{
	return m_language;
}

void CLanguageData::SetDir(char* sDir)
{
	m_dir = sDir;
}

void CLanguageData::SetLanguage(char* sLanguage)
{
	m_language = sLanguage;
}


