////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "CertDlg.h"
#include "SetupInfo.h"
#include "cert.h"
#include "CodeCountry.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC_CLASS(wxCertDlg, wxPanel)

enum
{
	IDC_COMMON = 10001,
	IDC_STATE,
	IDC_COUNTRY,
	IDC_EMAIL,
	IDC_ORGANIZATION,
	IDC_UNUT,
	IDC_FINGER,
	IDC_REGENERATE,
};

wxCertDlg::wxCertDlg()
{
}

wxCertDlg::~wxCertDlg()
{
}

wxCertDlg::wxCertDlg(wxWindow* parent)
{
	m_bCertificateChanged = false;
	Create(parent);
}

bool wxCertDlg::Create(wxWindow* parent)
{
	wxPanel::Create(parent, IDD_CERT, wxDefaultPosition, wxDefaultSize, wxNO_BORDER | wxTAB_TRAVERSAL);
	CreateControls();

	InitDialog();

	return true;
}

void wxCertDlg::CreateControls()
{
	wxBoxSizer* itemBoxSizer1 = new wxBoxSizer(wxVERTICAL);
	SetSizer(itemBoxSizer1);

	wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizer1->Add(itemBoxSizer3, 0, wxALIGN_LEFT | wxALL, 5);

	wxSize mine = wxDefaultSize;
	mine.SetWidth(150);

	//
	wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer4, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText5 = new wxStaticText(this, wxID_STATIC, _("Common name:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(itemStaticText5, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_Common = new wxTextCtrl(this, IDC_COMMON, _T(""), wxDefaultPosition, mine, 0);
	itemBoxSizer4->Add(m_Common, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	//
	wxBoxSizer* itemBoxSizer8 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer8, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText9 = new wxStaticText(this, wxID_STATIC, _("Country:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer8->Add(itemStaticText9, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_Country = new wxComboBox(this, IDC_COUNTRY, _T(""), wxDefaultPosition, mine, 0, NULL, wxCB_READONLY | wxCB_SORT);
	itemBoxSizer8->Add(m_Country, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	LoadCountry();
	//
	wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer7, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText8 = new wxStaticText(this, wxID_STATIC, _("State or province:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer7->Add(itemStaticText8, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_State = new wxTextCtrl(this, IDC_STATE, _T(""), wxDefaultPosition, mine, 0);
	itemBoxSizer7->Add(m_State, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	//
	wxBoxSizer* itemBoxSizer10 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer10, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText11 = new wxStaticText(this, wxID_STATIC, _("Email:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer10->Add(itemStaticText11, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_Email = new wxTextCtrl(this, IDC_EMAIL, _T(""), wxDefaultPosition, mine, 0);
	itemBoxSizer10->Add(m_Email, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	//
	wxBoxSizer* itemBoxSizer13 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer13, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText14 = new wxStaticText(this, wxID_STATIC, _("Organization:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer13->Add(itemStaticText14, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_Organization = new wxTextCtrl(this, IDC_ORGANIZATION, _T(""), wxDefaultPosition, mine, 0);
	itemBoxSizer13->Add(m_Organization, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	//
	wxBoxSizer* itemBoxSizer14 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer14, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText15 = new wxStaticText(this, wxID_STATIC, _("Unit:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer14->Add(itemStaticText15, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_Unit = new wxTextCtrl(this, IDC_UNUT, _T(""), wxDefaultPosition, mine, 0);
	itemBoxSizer14->Add(m_Unit, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	//
	wxBoxSizer* itemBoxSizer15 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer15, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText16 = new wxStaticText(this, wxID_STATIC, _("SHA1 Fingerprint:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer15->Add(itemStaticText16, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_Finger = new wxTextCtrl(this, IDC_FINGER, _T(""), wxDefaultPosition, mine, 0);
	itemBoxSizer15->Add(m_Finger, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	m_Finger->SetEditable(false);

	//
	wxBoxSizer* itemBoxSizer16 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer1->Add(itemBoxSizer16, 0, wxALIGN_LEFT | wxALL, 5);

	m_RegenrateOnStartUp = new wxCheckBox(this, IDC_REGENERATE, _("Regenerate SSL certificate on start-up"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer16->Add(m_RegenrateOnStartUp, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	//
}

void wxCertDlg::LoadSettings()
{
	m_Common->SetValue(g_info.GetCommon());

	SelectCountry(g_info.GetCountry());

	m_State->SetValue(g_info.GetState());
	m_Email->SetValue(g_info.GetEmail());
	m_Organization->SetValue(g_info.GetOrganization());
	m_Unit->SetValue(g_info.GetUnit());
	m_RegenrateOnStartUp->SetValue(g_info.GetRegenrateOnStartUp() != 0);
	m_Finger->SetValue(g_info.GetFinger());

	m_bCertificateChanged = false;
}

bool wxCertDlg::SaveSettings()
{
	wxString s = m_Common->GetValue();

	if(s.IsEmpty())
	{
		wxMessageBox(_("Common name cannot be empty."), _("Setup"));
		m_Common->SetFocus();
		return false;
	}

	if(s.Length() > 64)
	{
		wxMessageBox(_("Common name cannot be longer than 64 characters."), _("Setup"));
		m_Common->SetFocus();
		return false;
	}

	s = m_Country->GetValue();
	if(s.IsEmpty())
	{
		wxMessageBox(_("Country cannot be empty."), _("Setup"));
		m_Country->SetFocus();
		return false;
	}
	if(s.Length() > 64)
	{
		wxMessageBox(_("Country cannot be longer than 64 characters."), _("Setup"));
		m_Country->SetFocus();
		return false;
	}

	s = m_State->GetValue();
	if(s.IsEmpty())
	{
		wxMessageBox(_("State or province cannot be empty."), _("Setup"));
		m_State->SetFocus();
		return false;
	}
	if(s.Length() > 64)
	{
		wxMessageBox(_("State or province cannot be longer than 64 characters."), _T("Setup"));
		m_State->SetFocus();
		return false;
	}

	s = m_Email->GetValue();
	if(s.IsEmpty())
	{
		wxMessageBox(_("Email cannot be empty."), _T("Setup"));
		m_Email->SetFocus();
		return false;
	}
	if(s.Length() > 64)
	{
		wxMessageBox(_("Email cannot be longer than 64 characters."), _("Setup"));
		m_Email->SetFocus();
		return false;
	}

	s = m_Organization->GetValue();
	if(s.IsEmpty())
	{
		wxMessageBox(_("Organization cannot be empty."), _("Setup"));
		m_Organization->SetFocus();
		return false;
	}
	if(s.Length() > 64)
	{
		wxMessageBox(_("Organization cannot be longer than 64 characters."), _("Setup"));
		m_Organization->SetFocus();
		return false;
	}

	s = m_Unit->GetValue();
	if(s.IsEmpty())
	{
		wxMessageBox("Unit cannot be empty.");
		m_Unit->SetFocus();
		return false;
	}
	if(s.Length() > 64)
	{
		wxMessageBox(_("Unit cannot be longer than 64 characters."), _("Setup"));
		m_Unit->SetFocus();
		return false;
	}

	g_info.SetCommon(m_Common->GetValue());
	g_info.SetCountry(GetCountryCode());
	g_info.SetState(m_State->GetValue());
	g_info.SetEmail(m_Email->GetValue());
	g_info.SetOrganization(m_Organization->GetValue());
	g_info.SetUnit(m_Unit->GetValue());
	g_info.SetRegenrateOnStartUp(m_RegenrateOnStartUp->GetValue());

	return true;
}

BEGIN_EVENT_TABLE(wxCertDlg, wxPanel)
	EVT_TEXT(IDC_COMMON, wxCertDlg::OnCertificateChanged)
	EVT_TEXT(IDC_STATE, wxCertDlg::OnCertificateChanged)
	EVT_TEXT(IDC_COUNTRY, wxCertDlg::OnCertificateChanged)
	EVT_TEXT(IDC_EMAIL, wxCertDlg::OnCertificateChanged)
	EVT_TEXT(IDC_ORGANIZATION, wxCertDlg::OnCertificateChanged)
	EVT_TEXT(IDC_UNUT, wxCertDlg::OnCertificateChanged)
END_EVENT_TABLE()

void wxCertDlg::OnCertificateChanged(wxCommandEvent &)
{
	m_bCertificateChanged = true;
}

void wxCertDlg::LoadCountry()
{
	for(int i = 0; i < g_nCodeCountriesSize; i++)
	{
		int nIndex = m_Country->Append(g_arCodeCountries[i].m_pName);
		m_Country->SetClientData(nIndex, (void*)i);
	}
}

void wxCertDlg::SelectCountry(wxString sCode)
{
	for(int i = 0; i < g_nCodeCountriesSize; i++)
	{
		if(sCode == g_arCodeCountries[i].m_pCode)
		{
			m_Country->SetValue(g_arCodeCountries[i].m_pName);
		}
	}
}

wxString wxCertDlg::GetCountryCode()
{
	int nSel = m_Country->GetSelection();
	int nIndex = (int)m_Country->GetClientData(nSel);
	return g_arCodeCountries[nIndex].m_pCode;
}
