/**********************************************************************
*  File: DownloadFile.cpp
*
*  Purpose: Class for downloading a file from a given URL and send back 
*  event notification to the owner, when the download fails or complete.
*
*  Authors: Priyank Bolia <soft@priyank.in> 
*
*  Created: 24 December, 2005 
*
*  Modified: 12 January, 2006
*
*  Copyright (C) 2005-2006 by Priyank Bolia. All rights reserved.
*
*  Licence: wxWindows licence
* 
*  http://www.priyank.in
*********************************************************************/

#include "stdwx.h"
#include "DownloadFile.h"

wxDownloadFile::wxDownloadFile(wxWindow *pParent, wxString strURL, wxString strFile, bool bNotify, wxInt64 nBytes)
: wxThread(wxTHREAD_DETACHED)
, m_pParent(pParent)
, m_strURL(strURL)
, m_strFile(strFile)
, m_bIsDownload(true)
, m_nFileSize(0)
, m_bNotifyDownloading(bNotify)
, m_nNotifyBytes(nBytes)
{
	if ( this->Create() != wxTHREAD_NO_ERROR )
	{
		wxLogError(wxT("Can't create download thread!"));
	}
	else
	{
		if ( this->Run() != wxTHREAD_NO_ERROR )
		{
			wxLogError(wxT("Can't start download thread!"));
		}
	}
}

wxDownloadFile::~wxDownloadFile(void)
{
}

void* wxDownloadFile::Entry()
{
    int bytesread = 0;
	m_bIsDownload = true;
	wxDownloadEvent event( wxEVT_DOWNLOAD, GetId() );
	event.SetEventObject( (wxObject *)this->This() );
	event.SetDownLoadURL( m_strURL);
	event.SetDownLoadedFile( m_strFile);
	event.SetDownLoadStatus(DOWNLOAD_RUNNING);
	
	wxURL Url(m_strURL);
	((wxProtocol &)Url.GetProtocol()).SetTimeout(100);
	if (Url.GetError() == wxURL_NOERR)
	{
		wxInputStream *pIn_Stream = NULL;

#ifdef __WXMSW__
		wxRegKey *pRegKey = new wxRegKey(wxT("HKEY_CURRENT_USER\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"));
		if( pRegKey->Exists() && pRegKey->HasValue(wxT("ProxyEnable")))
		{
			long lProxyEnable;
			pRegKey->QueryValue(wxT("ProxyEnable"), &lProxyEnable);
			if(lProxyEnable == 1 && pRegKey->HasValue(wxT("ProxyServer")))
			{
				wxString strProxyAddress;
				pRegKey->QueryValue(wxT("ProxyServer"), strProxyAddress);
				Url.SetProxy(strProxyAddress);
			}
		}
		delete pRegKey;
#endif

		pIn_Stream = Url.GetInputStream();

		if(pIn_Stream)
		{
			m_nFileSize = pIn_Stream->GetSize();
			
			if(m_nFileSize != 0xFFFFFFFF)
				event.SetFileSize(m_nFileSize);
			
			wxFile file;
			wxInt64 nCount = 0;
			file.Create(m_strFile, true);

			char buff[1024];
			int nSize = 1024*sizeof(char);
			memset(buff, 0, nSize);

			while ((bytesread = (int)(pIn_Stream->Read(&buff, nSize)).LastRead()) > 0 && m_bIsDownload && !TestDestroy() )
			{
				file.Write((const void *)&buff, bytesread);
				memset(buff, 0, nSize);
	
				nCount += bytesread;
			
				if (m_bNotifyDownloading && (nCount%m_nNotifyBytes) == 0 && nCount >= m_nNotifyBytes) 
				{
					event.SetDownLoadStatus(DOWNLOAD_INPROGRESS);
					event.SetDownLoadedBytesCount(nCount);
					m_pParent->GetEventHandler()->ProcessEvent( event );
				}
			}
			file.Close();
			delete pIn_Stream;
			
			if(!m_bIsDownload)
			{
				event.SetDownLoadStatus(DOWNLOAD_CANCELLED);
			}
			else if(m_nFileSize != 0xFFFFFFFF && nCount != m_nFileSize)
			{
				event.SetDownLoadStatus(DOWNLOAD_FAIL);
			}
			else
			{
				event.SetDownLoadStatus(DOWNLOAD_COMPLETE);
			}
			
			if(m_pParent)
				m_pParent->GetEventHandler()->ProcessEvent(event);
		}
		else
		{
			event.SetDownLoadStatus(DOWNLOAD_FAIL);
			if(m_pParent)
				m_pParent->GetEventHandler()->ProcessEvent( event );
		}
	}
	else
	{
		event.SetDownLoadStatus(DOWNLOAD_FAIL);
		if(m_pParent)
			m_pParent->GetEventHandler()->ProcessEvent( event );
	}
	return 0;
}

void wxDownloadFile::OnExit()
{
}

void wxDownloadFile::CancelDownload(void)
{
	m_bIsDownload = false;
//	m_pParent = NULL;
}

wxInt64 wxDownloadFile::GetFileSize(void)
{
	return m_nFileSize;
}

void wxDownloadFile::SetDownloadingNotification(bool bEnable, wxInt64 nBytes)
{
	m_bNotifyDownloading = bEnable;
	m_nNotifyBytes = nBytes;
}
