////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "GetIP.h"

#define BUF_SIZE 4096

int GetIPFromURL(wxString& wxstrIP, const char* sURL)
{
	wxString exp = wxT("[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}");
	wxRegEx regex;
	regex.Compile(exp);
	int nRead = 0;

	wxURI uri(sURL);
	wxString sURLHost = uri.GetServer();
	wxString sURLPath = uri.GetPath();
	wxHTTP http;

	http.SetTimeout(10); //10 seconds better than default 10 mins!
	http.SetHeader(_T("Accept"), _T("text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5"));
	http.SetHeader(_T("Accept-Charset"), _T("ISO-8859-1,utf-8;q=0.7,*;q=0.7"));
	http.SetHeader(_T("Accept-Language"), _T("en-us,en;q=0.5"));
	http.SetHeader(_T("User-Agent"), _T("Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.8) Gecko/20061025 Firefox/1.5.0.8"));
	http.Connect(sURLHost, 80);

	wxInputStream* in_stream = http.GetInputStream(sURLPath);
	if(in_stream == NULL)
		return -1; //no data

	char buf[BUF_SIZE];    

	do
	{
		nRead = in_stream->Read(buf, BUF_SIZE).LastRead();
		wxString strContent = wxT(buf);

		if(regex.Matches(strContent))
		{
			wxstrIP = regex.GetMatch(strContent);

			delete in_stream;
			return 0; //good
		}
	}
	while(nRead == BUF_SIZE);

	delete in_stream;

	return -2; //didnt find ip
}

wxString GetLatestVersion()
{
	wxURL url(_T("http://psiphon.ca/download/version.txt"));

	if(url.GetError() != wxURL_NOERR)
		return _T(""); //no connection, no socket...

	wxInputStream* in_stream = url.GetInputStream();

	if(!in_stream)
		return _T(""); //no data

	char buf[10];    
	in_stream->Read(buf, 10).LastRead();
	delete in_stream;

	wxString sLatestVersion = wxT(buf);

	return sLatestVersion;
}

int TestPort(int nPort)
{
	wxIPV4address addr;
	addr.Service(nPort);

	wxSocketServer server(addr);

	if(server.Ok())
		return 0;

	return -1;
}
