////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "LinkDlg.h"
#include "SetupInfo.h"
#include "GetIP.h"
#include "Server.h"
#include "PsiphonDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC_CLASS(wxLinkDlg, wxPanel)

enum
{
	IDC_IP = 10001, 
	IDC_TEST_PORT,
	IDC_PORT,	 
	IDC_LINK,	 
	IDC_UPDATE_IP,  
	IDC_AUTO_UPDATE_IP, 
	IDC_SET_PORT,	
	IDC_COPY_TO_CLIPBOARD,	
	IDC_NAME,		
	IDC_GETIP_SERVERS, 
};

wxLinkDlg::wxLinkDlg()
{
}

wxLinkDlg::~wxLinkDlg()
{
}

wxLinkDlg::wxLinkDlg(wxPsiphonDlg* _pPsiphonDlg, wxWindow* parent)
{
	m_pPsiphonDlg = _pPsiphonDlg;
	m_bLinkChanged = false;
	Create(parent);
}

bool wxLinkDlg::Create(wxWindow* parent)
{
	wxPanel::Create(parent, IDD_SETTINGS);
	CreateControls();

	InitDialog();

	return true;
}

void wxLinkDlg::CreateControls()
{
	wxBoxSizer* itemBoxSizerMainV = new wxBoxSizer(wxVERTICAL);
	SetSizer(itemBoxSizerMainV);

	wxBoxSizer* itemBoxSizerMainV2 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizerMainV->Add(itemBoxSizerMainV2, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

//IP
	wxBoxSizer* itemBoxSizerIP = new wxBoxSizer(wxVERTICAL);
	itemBoxSizerMainV2->Add(itemBoxSizerIP, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticText1 = new wxStaticText(this, wxID_STATIC, _("Your psiphonite users need your external IP address\n"
																		"in order to connect to psiphon."), wxDefaultPosition, wxDefaultSize, 0);

	itemBoxSizerIP->Add(itemStaticText1, 0, wxALIGN_LEFT | wxALL, 5);

	//External IP	
	wxBoxSizer* itemBoxSizerIPEnter = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerIP->Add(itemBoxSizerIPEnter, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticTextPort = new wxStaticText(this, wxID_STATIC, _("External IP:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerIPEnter->Add(itemStaticTextPort, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_IP = new wxTextCtrl(this, IDC_IP, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	itemBoxSizerIPEnter->Add(m_IP, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	wxButton* itemButtonIP = new wxButton(this, IDC_UPDATE_IP, _("&Update"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerIPEnter->Add(itemButtonIP, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	
	//Get IP Server
	wxBoxSizer* itemBoxGetIPServer = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerIP->Add(itemBoxGetIPServer, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticText14 = new wxStaticText(this, wxID_STATIC, _("Server:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxGetIPServer->Add(itemStaticText14, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	wxSize mine = wxDefaultSize;
	mine.SetWidth(200);

	m_GetIPServers = new wxComboBox(this, IDC_GETIP_SERVERS, _T(""), wxDefaultPosition, mine, 0, NULL);
	itemBoxGetIPServer->Add(m_GetIPServers, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	//Update IP
	m_AutoUpdateIP = new wxCheckBox(this, IDC_AUTO_UPDATE_IP, _("Update external IP on start-up"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerIP->Add(m_AutoUpdateIP, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

//PORT
	wxBoxSizer* itemBoxSizerPORT = new wxBoxSizer(wxVERTICAL);
	itemBoxSizerMainV2->Add(itemBoxSizerPORT, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticText3 = new wxStaticText(this, wxID_STATIC, _("Verify that the psiphon port is available by clicking on 'Test'."), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerPORT->Add(itemStaticText3, 0, wxALIGN_LEFT | wxALL | wxADJUST_MINSIZE, 5);
	//
	wxBoxSizer* itemBoxSizerPORTEnter = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerPORT->Add(itemBoxSizerPORTEnter, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticText4 = new wxStaticText(this, wxID_STATIC, _("Port (1-65535):"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerPORTEnter->Add(itemStaticText4, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_PORT = new wxTextCtrl(this, IDC_PORT, _T(""), wxDefaultPosition, wxSize(40, -1), 0);
	itemBoxSizerPORTEnter->Add(m_PORT, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	wxButton* _TestButton = new wxButton(this, IDC_TEST_PORT, _("&Test"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerPORTEnter->Add(_TestButton, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	//
	m_AutoTestPORT = new wxCheckBox(this, IDC_AUTO_UPDATE_IP, _("Test this psiphon port on start-up"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerPORT->Add(m_AutoTestPORT, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

//server name
	wxBoxSizer* itemBoxSizerNAME = new wxBoxSizer(wxVERTICAL);
	itemBoxSizerMainV2->Add(itemBoxSizerNAME, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticText5 = new wxStaticText(this, wxID_STATIC, _("Please name your psiphon (a-Z, 0-9)."), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerNAME->Add(itemStaticText5, 0, wxALIGN_LEFT | wxALL | wxADJUST_MINSIZE, 5);
	//
	wxBoxSizer* itemBoxSizerNAMEEnter = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerNAME->Add(itemBoxSizerNAMEEnter, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticText6 = new wxStaticText(this, wxID_STATIC, _("Name:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerNAMEEnter->Add(itemStaticText6, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_NAME = new wxTextCtrl(this, IDC_NAME, _T(""), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerNAMEEnter->Add(m_NAME, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	//
}

void wxLinkDlg::LoadSettings()
{
	m_IP->SetValue(g_info.GetIP());
	m_AutoUpdateIP->SetValue(g_info.GetAutoUpdateIP() != 0);

	m_PORT->SetValue(wxString::Format(_T("%d"), g_info.GetPort()));
	m_AutoTestPORT->SetValue(g_info.GetTestPort() != 0);

	m_NAME->SetValue(g_info.GetName());

	m_bLinkChanged = false;

	m_GetIPServers->AppendString(_T("http://psiphon.civisec.org/ip.php"));
	m_GetIPServers->AppendString(_T("http://whatismyip.com/"));
	m_GetIPServers->AppendString(_T("http://showmyip.com/"));
	m_GetIPServers->AppendString(_T("http://checkip.dyndns.org/"));
	m_GetIPServers->SetValue(g_info.GetIPServer());
}

bool IsAlphaNumeric(wxString sName)
{
	for(size_t i = 0; i < sName.Length(); i++)
	{
		char ch = sName.GetChar(i);

		if(!isalnum(ch))
			return false;
	}

	return true;
}

bool wxLinkDlg::SaveSettings()
{
	wxString sIP = m_IP->GetValue();
	int nPort = atoi(m_PORT->GetValue());
	wxString sName = m_NAME->GetValue();

	if(nPort <= 0 || nPort >= 65536)
	{
		wxMessageBox(_("psiphon port is invalid."), _("Setup"));
		return false;
	}

	if(sName.IsEmpty())
	{
		wxMessageBox(_("psiphon name cannot be empty."), _("Setup"));
		return false;
	}

	if(!IsAlphaNumeric(sName))
	{
		wxMessageBox(_("psiphon name is invalid."), _("Setup"));
		return false;
	}

	g_info.SetIP(sIP);	
	g_info.SetAutoUpdateIP(m_AutoUpdateIP->GetValue());

	g_info.SetPort(nPort);
	g_info.SetTestPort(m_AutoTestPORT->GetValue());

	g_info.SetName(sName);

	g_info.SetIPServer(m_GetIPServers->GetValue());

	return true;
}

BEGIN_EVENT_TABLE(wxLinkDlg, wxPanel)
	EVT_BUTTON(IDC_UPDATE_IP, wxLinkDlg::OnUpdateIP)
	EVT_BUTTON(IDC_TEST_PORT, wxLinkDlg::OnTestPort)
	EVT_TEXT(IDC_IP, wxLinkDlg::OnLinkChange)
	EVT_TEXT(IDC_PORT, wxLinkDlg::OnLinkChange)
	EVT_TEXT(IDC_NAME, wxLinkDlg::OnLinkChange)
END_EVENT_TABLE()

void wxLinkDlg::OnUpdateIP(wxCommandEvent &)
{
	wxBusyCursor wait;

	wxString sOldIP;
	sOldIP = m_IP->GetValue();
	m_IP->SetValue(_("(updating...)"));

	wxString sIP;
	int nRes = GetIPFromURL(sIP, m_GetIPServers->GetValue());

	if(nRes == 0)
	{
		m_IP->SetValue(sIP);
	}
	else if(nRes == -1)
	{
		wxMessageBox(_("psiphon failed to connect to server."), _("Setup"));
		m_IP->SetValue(sOldIP);
	}
	else
	{
		wxMessageBox(_("psiphon failed to retrieve ip from server."), _("Setup"));
		m_IP->SetValue(sOldIP);
	}
}

void wxLinkDlg::OnTestPort(wxCommandEvent&)
{
	int nPort = atoi(m_PORT->GetValue());

	if(nPort <= 0 || nPort >= 65536)
	{
		wxMessageBox(_("psiphon port is invalid."), _("Setup"));
		return;
	}

	bool bRestartServer = g_info.GetPort() == nPort && m_pPsiphonDlg->IsServerRunning();

	if(bRestartServer)
	{
		if(m_pPsiphonDlg->StopServer() == false)
			return;
	}

	int nRes = TestPort(nPort);

	if(bRestartServer)
		m_pPsiphonDlg->StartServer();

	if(nRes != 0)
	{
		wxString sMess = wxString::Format(_("The port %d is not available.\nSelect another port number and re-test."), nPort);

		wxMessageBox(sMess, _("Setup"));
	}
	else
	{
		wxString sMess = wxString::Format(_("The port %d is available."), nPort);
		wxMessageBox(sMess, _("Setup"));
	}
}

void wxLinkDlg::OnLinkChange(wxCommandEvent&)
{
	m_bLinkChanged = true;
}
