////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "LogDlg.h"
#include "PsiphonLog.h"
#include "PsiphonDlg.h"
#include "SetupInfo.h"
#include "strfunc.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC_CLASS(wxLogDlg, wxPanel)

enum
{
	IDC_LOG_LISTCTRL = 10001,
	IDC_LEVEL,
	IDC_CLEANUP,
};

void wxLogDlg::Init()
{
	m_loglist = NULL;
}

wxLogDlg::wxLogDlg()
{
	Init();
}

wxLogDlg::wxLogDlg(wxPsiphonDlg* _pPsiphonDlg, wxWindow* parent)
{
	Init();
	m_pPsiphonDlg = _pPsiphonDlg;
	Create(parent);
}

bool wxLogDlg::Create(wxWindow* parent)
{
	wxPanel::Create(parent, IDD_LOG);

	CreateControls();
	if(GetSizer())
		GetSizer()->Fit(this);

	InitDialog();

	return true;
}

void wxLogDlg::CreateControls()
{
	wxLogDlg* itemPanel1 = this;

	wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
	itemPanel1->SetSizer(itemBoxSizer2);

	m_loglist = new wxListCtrlSort(itemPanel1, IDC_LOG_LISTCTRL, wxDefaultPosition, wxDefaultSize, wxLC_REPORT);
	m_loglist->InsertColumn(CLMN_MESSAGE, _("Message"), wxLIST_FORMAT_LEFT, 200);
	m_loglist->InsertColumn(CLMN_USER_ID, _("User ID"), wxLIST_FORMAT_LEFT, 60, true);
	m_loglist->InsertColumn(CLMN_TIME, _("Date added"), wxLIST_FORMAT_LEFT, 120);

	itemBoxSizer2->Add(m_loglist, -1, wxGROW | wxALL, 5);

	wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer2->Add(itemBoxSizer4, 0, wxALIGN_RIGHT | wxALL, 5);

	wxStaticText* itemStaticText5 = new wxStaticText(itemPanel1, wxID_STATIC, _("Log level:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(itemStaticText5, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	m_cmbLevel = new wxComboBox(itemPanel1, IDC_LEVEL, _T(""), wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY);
	itemBoxSizer4->Add(m_cmbLevel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	itemBoxSizer4->Add(5, 5, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	wxButton* itemButton8 = new wxButton(itemPanel1, IDC_CLEANUP, _("&Cleanup"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(itemButton8, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
}

void wxLogDlg::Log(const wxString& msg, int nUserID)
{
	long nIndex = m_loglist->InsertItem(m_loglist->GetItemCount(), _T(""));
	m_loglist->SetItemData(nIndex, nIndex);

	wxString sUserID;
	if(nUserID >= 0)
		sUserID = wxString::Format(_T("%d"), nUserID);

	wxDateTime dt(wxDateTime::GetTimeNow());
	wxString sDateTime = dt.Format(_T("%Y-%m-%d %H:%M:%S"));

	m_loglist->SetItem(nIndex, CLMN_MESSAGE, msg);
	m_loglist->SetItem(nIndex, CLMN_USER_ID, sUserID);
	m_loglist->SetItem(nIndex, CLMN_TIME, sDateTime);

	m_loglist->EnsureVisible(nIndex);

	wxString s = wxString::Format(_T("%s\t%s\t%s"), sUserID.c_str(), sDateTime.c_str(), msg.c_str());
	LogItToFile(s);

	if(m_loglist->GetItemCount() > 1000)
	{
		Cleanup();
		long nIndex = m_loglist->InsertItem(m_loglist->GetItemCount(), _T(""));
		m_loglist->SetItem(nIndex, CLMN_MESSAGE, _("(previous log info was moved to a log file)"));
	}
}

void wxLogDlg::Cleanup()
{
	m_loglist->DeleteAllItems();
}

void wxLogDlg::LogItToFile(const wxString& msg)
{
	wxFileName::Mkdir(_T("log"), 0777, wxPATH_MKDIR_FULL);
	wxDateTime dt(wxDateTime::GetTimeNow());
	wxString sFileName = dt.Format(_T("log/%Y-%m-%d.txt"));

	wxFile file;
	file.Open(sFileName, wxFile::write_append, 0777);

	file.Write(msg);

	file.Write(_T("\n"));

	file.Close();
}

BEGIN_EVENT_TABLE(wxLogDlg, wxPanel)
	EVT_INIT_DIALOG(wxLogDlg::OnInitDialog) 
	EVT_BUTTON(IDC_CLEANUP, wxLogDlg::OnCleanup)
	EVT_COMBOBOX(IDC_LEVEL, wxLogDlg::OnLevelChanged) 
	EVT_LIST_ITEM_ACTIVATED(IDC_LOG_LISTCTRL, wxLogDlg::OnListItemActivated) 
END_EVENT_TABLE()

#define LEVEL_NONE _("(none)")

void wxLogDlg::OnInitDialog(wxInitDialogEvent&)
{
	m_cmbLevel->AppendString(LEVEL_NONE);
	m_cmbLevel->AppendString(_T("0"));
	m_cmbLevel->AppendString(_T("1"));
	m_cmbLevel->AppendString(_T("2"));
	m_cmbLevel->AppendString(_T("3"));
	m_cmbLevel->AppendString(_T("4"));
	m_cmbLevel->AppendString(_T("5"));
	m_cmbLevel->AppendString(_T("6"));
	m_cmbLevel->AppendString(_T("7"));
	m_cmbLevel->AppendString(_T("8"));
	m_cmbLevel->AppendString(_T("9"));

	wxString sDefaultLogLevel;
	int nLogLevel = g_info.GetLogLevel();

	if(nLogLevel == -1)
		sDefaultLogLevel = LEVEL_NONE;
	else
		sDefaultLogLevel = wxString::Format(_T("%d"), nLogLevel);

	m_cmbLevel->SetValue(sDefaultLogLevel);
}

void wxLogDlg::OnLevelChanged(wxCommandEvent&)
{
	int nLevel = 0;
	wxString sLevel = m_cmbLevel->GetValue();

	if(sLevel == LEVEL_NONE)
	{
		nLevel = -1;
	}
	else
	{
		nLevel = atoi(sLevel);
	}

	g_info.SetLogLevel(nLevel);
	g_info.Store();

	m_pPsiphonDlg->RestartServer();
}

void wxLogDlg::OnCleanup(wxCommandEvent&)
{
	Cleanup();
}

void wxLogDlg::OnListItemActivated(wxListEvent&)
{	
	int nIndex = m_loglist->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

	if(nIndex < 0)
		return;

	wxString sText = m_loglist->GetText(nIndex, CLMN_MESSAGE);

	if(!StartsWith(sText.c_str(), _T("http://")))
		return;

	wxLaunchDefaultBrowser(sText);
}

