////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "PrefDlg.h"
#include "SetupInfo.h"
#include "GetIP.h"
#include "UpdateDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC_CLASS(wxPrefDlg, wxPanel)

enum
{
	IDC_SHOW_IMAGES = 10001,
	IDC_DONT_SHOW_SETUP,
	IDC_START_ON_STARTUP,
	IDC_CHECK_UPDATES_ON_STARTUP,
	IDC_CHECK_FOR_UPDATES_NOW,
};

wxPrefDlg::wxPrefDlg()
{
}

wxPrefDlg::~wxPrefDlg()
{
}

wxPrefDlg::wxPrefDlg(wxWindow* parent)
{
	Create(parent);
}

bool wxPrefDlg::Create(wxWindow* parent)
{
	wxPanel::Create(parent, IDD_PREF);
	CreateControls();

	InitDialog();

	return true;
}

void wxPrefDlg::CreateControls()
{
	wxBoxSizer* itemBoxSizerMainV = new wxBoxSizer(wxVERTICAL);
	SetSizer(itemBoxSizerMainV);

	wxBoxSizer* itemBoxSizerMainV2 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizerMainV->Add(itemBoxSizerMainV2, 0, wxALIGN_LEFT | wxALL, 5);

	wxBoxSizer* itemBoxSizerMainV3 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizerMainV2->Add(itemBoxSizerMainV3, 0, wxALIGN_LEFT | wxALL, 5);
//
	wxBoxSizer* itemBoxSizerUpdates = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerMainV2->Add(itemBoxSizerUpdates, 0, wxALIGN_LEFT | wxALL, 5);

	m_CheckForUpdatesOnStartUp = new wxCheckBox(this, IDC_CHECK_UPDATES_ON_STARTUP, _("Check for updates on start-up"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerUpdates->Add(m_CheckForUpdatesOnStartUp, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	wxButton* itemButton12 = new wxButton(this, IDC_CHECK_FOR_UPDATES_NOW, _("&Check for updates..."), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerUpdates->Add(itemButton12, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
//
	wxBoxSizer* itemBoxSizerStart = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerMainV2->Add(itemBoxSizerStart, 0, wxALIGN_LEFT | wxALL, 5);

	m_StartOnStartUp = new wxCheckBox(this, IDC_START_ON_STARTUP, _("Start psiphon on start-up"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerStart->Add(m_StartOnStartUp, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
//
	wxBoxSizer* itemBoxSizerImages = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerMainV2->Add(itemBoxSizerImages, 0, wxALIGN_LEFT | wxALL, 5);

	m_ShowImages = new wxCheckBox(this, IDC_SHOW_IMAGES, _("Allow psiphonites to view images"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerImages->Add(m_ShowImages, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
}

void wxPrefDlg::LoadSettings()
{
	m_ShowImages->SetValue(g_info.GetShowImages() != 0);
	m_StartOnStartUp->SetValue(g_info.GetStartOnStartUp() != 0);
	m_CheckForUpdatesOnStartUp->SetValue(g_info.GetCheckForUpdatesOnStartUp() != 0);
}

bool wxPrefDlg::SaveSettings()
{
	g_info.SetShowImages(m_ShowImages->GetValue());
	g_info.SetStartOnStartUp(m_StartOnStartUp->GetValue());
	g_info.SetCheckForUpdatesOnStartUp(m_CheckForUpdatesOnStartUp->GetValue());

	return true;
}

BEGIN_EVENT_TABLE(wxPrefDlg, wxPanel)
	EVT_BUTTON(IDC_CHECK_FOR_UPDATES_NOW, wxPrefDlg::OnCheckForUpdates)
END_EVENT_TABLE()

void wxPrefDlg::OnCheckForUpdates(wxCommandEvent&)
{
	wxUpdateDlg::CheckForUpdates(this, true);
}

