////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "UserDlg.h"
#include "LogDlg.h"
#include "Server.h"
#include "PsiphonLog.h"
#include "PsiphonDlg.h"
#include "PsiphonApp.h"
#include "logDetails.h"
#include "SetupDlg.h"
#include "SetupInfo.h"
#include "GetIP.h"
#include "PsiphonNameDlg.h"
#include "cert.h"
#include "strfunc.h"
#include "psiphonon.xpm"
#include "psiphonoff.xpm"
#include "taskbaricon.h"
#include "UpdateDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC_CLASS(wxPsiphonDlg, wxDialog)

enum
{
	IDC_TABS = 10001,
	IDC_STOP,
	IDC_START,
	IDC_SETUP,
	IDC_EXIT,
	IDC_HYPERLINK,
};


DEFINE_EVENT_TYPE(wxEVT_RUN_START_UPS)

wxPsiphonDlg::wxPsiphonDlg()
{
}

wxPsiphonDlg::wxPsiphonDlg(wxWindow* parent)
#ifdef _WIN32
			:m_icoON(wxICON(psiphonon))
			,m_icoOFF(wxICON(psiphonoff))
#else
			:m_icoON(psiphonon_xpm)
			,m_icoOFF(psiphonoff_xpm)
#endif
{
	m_taskBarIcon = NULL;
	Create(parent);
}

wxPsiphonDlg::~wxPsiphonDlg()
{
	delete m_taskBarIcon;
}

bool wxPsiphonDlg::Create(wxWindow* parent)
{
	SetExtraStyle(GetExtraStyle() | wxWS_EX_BLOCK_EVENTS);
	wxDialog::Create(parent, IDD_PSIPHON, _("psiphon"), wxDefaultPosition, wxSize(600, 400), wxRESIZE_BORDER|wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxMINIMIZE_BOX|wxMAXIMIZE_BOX);

	CreateControls();

	m_taskBarIcon = new CPsiphonTaskBarIcon(this);

	if(GetSizer())
	{
		GetSizer()->SetSizeHints(this);
	}

	Centre();
	
	SetIconON(false);

	return true;
}

void wxPsiphonDlg::CreateControls()
{
	wxPsiphonDlg* itemDialog1 = this;

	wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
	itemDialog1->SetSizer(itemBoxSizer2);

	wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizer2->Add(itemBoxSizer3, 1, wxGROW | wxALL, 5);

	//hyper link
	wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizer4, 0, wxALIGN_LEFT | wxALL, 5);

	wxStaticText* itemStaticText8 = new wxStaticText(itemDialog1, wxID_STATIC, _("Link:"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(itemStaticText8, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	wxString sLink = _T("https://127.0.0.1:443/login/");

	wxSize mine = wxDefaultSize;
	mine.SetWidth(250);

	m_Hyperlink = new wxHyperlinkCtrl(this, IDC_HYPERLINK, sLink, sLink, wxDefaultPosition, mine, wxHL_CONTEXTMENU | wxNO_BORDER | wxHL_ALIGN_LEFT);
	itemBoxSizer4->Add(m_Hyperlink, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	//tabs
	wxNotebook* itemNotebook4 = new wxNotebook(itemDialog1, IDC_TABS, wxDefaultPosition, wxDefaultSize, wxNB_TOP);
	itemBoxSizer3->Add(itemNotebook4, 1, wxGROW | wxALL, 5);

	m_pUserDlg = new wxUserDlg(itemNotebook4);
	itemNotebook4->AddPage(m_pUserDlg, _("Users"));

	m_pLogDlg = new wxLogDlg(this, itemNotebook4);
	itemNotebook4->AddPage(m_pLogDlg, _("Log"));

	//buttons
	wxBoxSizer* itemBoxSizer21 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizer2->Add(itemBoxSizer21, 0, wxGROW | wxRIGHT | wxTOP | wxBOTTOM, 5);

	m_btnStart = new wxButton(itemDialog1, IDC_START, _("&Start"), wxDefaultPosition, wxDefaultSize, 0);
	m_btnStart->SetDefault();
	itemBoxSizer21->Add(m_btnStart, 0, wxGROW | wxALL, 5);

	m_btnStop = new wxButton(itemDialog1, IDC_STOP, _("S&top"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer21->Add(m_btnStop, 0, wxGROW | wxALL, 5);

	wxButton* itemButton24 = new wxButton(itemDialog1, IDC_SETUP, _("S&etup..."), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer21->Add(itemButton24, 0, wxGROW | wxALL, 5);

	wxButton* itemButton25 = new wxButton(itemDialog1, IDC_EXIT, _("E&xit"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer21->Add(itemButton25, 0, wxGROW | wxALL, 5); 
}

void wxPsiphonDlg::SetIconON(bool bON)
{
	if(bON)
	{
	 	SetIcon(m_icoON);
		m_taskBarIcon->SetIcon(m_icoON, GetTitle());
	}
	else
	{
	 	SetIcon(m_icoOFF);
		m_taskBarIcon->SetIcon(m_icoOFF, GetTitle());
	}
}

bool wxPsiphonDlg::IsServerRunning()
{
	return m_server.IsServerRunning();
}

void wxPsiphonDlg::StartServer()
{
	wxBusyCursor wait;

	if(m_server.IsServerRunning())
		return;

	SetTitleStatus(_("Trying to start..."));

	m_server.m_config.m_nLogLevel = g_info.GetLogLevel();

	m_server.m_config.m_nPort = g_info.GetPort();

	delete m_server.m_config.m_logFunc;
	m_server.m_config.m_logFunc = new wxPsiphonLog(m_server.m_config.m_nLogLevel);
	m_server.m_config.m_crashFunc = wxPsiphonLog::postServerCrashedMessage;

	bool bRes = m_server.Start();

	if(bRes == false)
	{
		wxMessageBox(_("psiphon failed to start.\n"
						"Go to Setup to make sure your settings are correct."), _("psiphon"));

		PsiphonIsOFF();
		return;
	}

	PsiphonIsON();
}

void wxPsiphonDlg::PsiphonIsOFF()
{
	SetTitleStatus(_("Stopped"));
	m_btnStart->Enable(true);
	m_btnStop->Enable(false);

	SetIconON(false);
}

void wxPsiphonDlg::PsiphonIsON()
{
	SetTitleStatus(_("Started"));
	m_btnStart->Enable(false);
	m_btnStop->Enable(true);

	SetIconON(true);
}

bool wxPsiphonDlg::StopServer(bool bSilent)
{
	if(!m_server.IsServerRunning())
		return true;

	SetTitleStatus(_("Trying to stop..."));

	bool bRes = m_server.Stop();

	if(bRes == false)
	{
		if(bSilent == false)
		{
			wxMessageBox(_("psiphon failed to stop.\n"
						"psiphon might need to be restarted."), _("psiphon"));
		}

		PsiphonIsON();
		return false;
	}

	PsiphonIsOFF();
	return true;
}

void wxPsiphonDlg::SetLink(wxString sLink)
{
	m_Hyperlink->SetURL(sLink);
	m_Hyperlink->SetLabel(sLink);	
	m_Hyperlink->Refresh();
}

void wxPsiphonDlg::UpdateLink()
{
	wxString sLink;

	if(g_info.GetName() != NULL)
		sLink = wxString::Format(_T("https://%s:%d/%s/"), g_info.GetIP(), g_info.GetPort(), g_info.GetName());
	else
		sLink = wxString::Format(_T("https://%s:%d/"), g_info.GetIP(), g_info.GetPort());

	SetLink(sLink);
}

void wxPsiphonDlg::Setup()
{
	wxSetupDlg dlg(this);

	if(dlg.ShowModal() != wxID_OK)
		return;

	g_info.Store();

	if(dlg.IsCertificateChanged() || dlg.IsLinkChanged())
	{
		if(dlg.IsCertificateChanged())
			InstallCertificate(true);

		RestartServer();
	}

	UpdateLink();
}

void wxPsiphonDlg::Exit()
{
	StopServer(true);
	Destroy();
}

void wxPsiphonDlg::RestartServer()
{
	if(!m_server.IsServerRunning())
		return;

	if(StopServer() == false)
		return;

	StartServer();
}

wxString wxPsiphonDlg::SetTitleStatus(wxString sStatus)
{
	wxString sOldStatus = m_sCurrentStatus;
	m_sCurrentStatus = sStatus;
	wxString sTitle = wxString::Format(_T("%s v%s - %s"), _("psiphon"), PSIPHON_VERSION, m_sCurrentStatus.c_str());
	SetTitle(sTitle);
	return sOldStatus;
}

bool wxPsiphonDlg::CheckForUpdatesOnStartup()
{
	wxString sTitle = _("Checking for updates on start-up...");
	wxString sOldTitleStatus = SetTitleStatus(sTitle);

	bool bInternetPass = wxUpdateDlg::CheckForUpdates(this, false);
	
	if(!bInternetPass)
		wxMessageBox(_("psiphon failed to connect to server."), sTitle);

	SetTitleStatus(sOldTitleStatus);

	return bInternetPass;
}

bool wxPsiphonDlg::UpdateName()
{
	wxPsiphonNameDlg dlg(this);

	if(dlg.ShowModal() != wxID_OK)
	{
		Destroy();  //AFF psiphon name must be specified
		return false;
	}

	g_info.SetName(dlg.GetName());
	g_info.Store();

	return true;
}

bool wxPsiphonDlg::AutoUpdateIP()
{
	wxString sTitle = _("Updating external IP on start-up...");
	wxString sOldStatus = SetTitleStatus(sTitle);

	wxString sOldIP = g_info.GetIP();

	wxString sNewIP;
	int nRes = GetIPFromURL(sNewIP, g_info.GetIPServer());

	bool bInternetPass = true;

	if(nRes == 0)
	{
		if(!sOldIP.IsEmpty() && sNewIP != sOldIP)
			wxMessageBox(_("Your external IP changed. Please notify your psiphonites."), sTitle);

		g_info.SetIP(sNewIP);
		g_info.Store();
	}
	else if(nRes == -1)
	{
		wxMessageBox(_("psiphon failed to connect to server."), sTitle);
		bInternetPass = false;
	}
	else
	{
		wxMessageBox(_("psiphon failed to retrieve ip from server."), sTitle);
		bInternetPass = false;
	}

	UpdateLink();

	SetTitleStatus(sOldStatus);

	return bInternetPass;
}

bool wxPsiphonDlg::FindAvailablePort()
{
	for(int i = 0; i < 100; i++)
	{
        srand(time(NULL));
		int nPort = rand()%65535;
	
		if(TestPort(nPort) == 0)
		{
			g_info.SetPort(nPort);
			g_info.Store();

			return true;
		}
	}

	return false;
}

bool wxPsiphonDlg::AutoTestPort()
{
	wxString sTitle = _("Testing psiphon port on start-up...");
	wxString sOldStatus = SetTitleStatus(sTitle);

	bool bPortPass = true;
	int nPort = g_info.GetPort();

	if(TestPort(nPort) != 0)
	{
		wxString sQuestion = wxString::Format(_("Port %d is not available.\nDo you want to find another available port?"), nPort);	
		if(wxMessageBox(sQuestion, sTitle, wxYES_NO | wxICON_QUESTION) == wxYES)
		{
			if(FindAvailablePort() == false)
				bPortPass = false;
		}
		else
		{
			bPortPass = false;
		}
	}

	UpdateLink();
	SetTitleStatus(sOldStatus);

	return bPortPass;
}

void wxPsiphonDlg::AutoInstallCertificate()
{
	wxString sOldTitleStatus = SetTitleStatus(_("Installing SSL Certificate..."));
	InstallCertificate(true);
	SetTitleStatus(sOldTitleStatus);
}

void wxPsiphonDlg::Hide()
{
	wxDialogBase::Show(false);
}

void wxPsiphonDlg::Restore()
{
	wxDialogBase::Show(true);
}

BEGIN_EVENT_TABLE(wxPsiphonDlg, wxDialog)
	EVT_INIT_DIALOG(wxPsiphonDlg::OnInitDialog) 

	EVT_COMMAND(-1, wxEVT_LOG, wxPsiphonDlg::OnEventLog)
	EVT_COMMAND(-1, wxEVT_SERVER_CRASHED, wxPsiphonDlg::OnServerCrashed)
	EVT_COMMAND(-1, wxEVT_RUN_START_UPS, wxPsiphonDlg::OnRunStartUps)

	EVT_BUTTON(IDC_START, wxPsiphonDlg::OnStartServer)
	EVT_BUTTON(IDC_STOP, wxPsiphonDlg::OnStopServer)
	EVT_BUTTON(IDC_SETUP, wxPsiphonDlg::OnSetup)
	EVT_BUTTON(IDC_EXIT, wxPsiphonDlg::OnExit)

	EVT_CLOSE(wxPsiphonDlg::OnCloseWindow) 
END_EVENT_TABLE()

void wxPsiphonDlg::OnInitDialog(wxInitDialogEvent &)
{
	PsiphonIsOFF();

	wxCommandEvent event(wxEVT_RUN_START_UPS);
	wxPostEvent(wxTheApp->GetTopWindow(), event);
}

void wxPsiphonDlg::OnCloseWindow(wxCloseEvent&)
{
	Hide();
}

void wxPsiphonDlg::OnStartServer(wxCommandEvent&)
{
	if(m_server.IsServerRunning())
	{
		wxMessageBox(_("psiphon is already on."), _("psiphon"));
		return;
	}

	StartServer();
}

void wxPsiphonDlg::OnStopServer(wxCommandEvent&)
{
	if(!m_server.IsServerRunning())
	{
		wxMessageBox(_("psiphon is already off."), _("psiphon"));
		return;
	}

	StopServer();
}

void wxPsiphonDlg::OnRunStartUps(wxCommandEvent&)
{
	wxBusyCursor wait;

	UpdateLink();

	if(g_info.GetName() == NULL)
	{
		if(UpdateName() == false)
			return;
	}

	bool bInternetPass = true;
	bool bPortPass = true;

	if(g_info.GetCheckForUpdatesOnStartUp())
	{
		bInternetPass = CheckForUpdatesOnStartup();
	}
	if(bInternetPass && g_info.GetAutoUpdateIP() != 0)
	{
		bInternetPass = AutoUpdateIP();
	}
	if(g_info.GetTestPort() != 0)
	{
		bPortPass = AutoTestPort();
	}

	if(g_info.GetRegenrateOnStartUp())
	{
		AutoInstallCertificate();
	}

	if(bPortPass && bInternetPass && g_info.GetStartOnStartUp()) //AFF there is no point in trying to start server ip port failed
	{
		wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, IDC_START);
		wxPostEvent(wxTheApp->GetTopWindow(), event);
	}
}

void wxPsiphonDlg::OnEventLog(wxCommandEvent& evt)
{
	wxString sLog = evt.GetString();

	CPsiphonMessage pm;

	if(pm.Parse(sLog))  //our message
	{
		if(strcmp(pm.GetMessage(), PSIPHON_MESSAGE_REQUEST) == 0)
		{
			if(m_pUserDlg)
				m_pUserDlg->Log(&pm);

			if(m_pLogDlg)
				m_pLogDlg->Log(pm.GetURI(), pm.GetUserID());
		}
	}
	else //appweb message
	{
		if(m_pLogDlg)
			m_pLogDlg->Log(sLog);
	}
}

void wxPsiphonDlg::OnServerCrashed(wxCommandEvent&)
{
	wxGetApp().Restart();
}


void wxPsiphonDlg::OnSetup(wxCommandEvent&)
{
	Setup();
}

void wxPsiphonDlg::OnExit(wxCommandEvent&)
{
	Exit();
}

