////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "Server.h"
#include "strfunc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CAppWebServer::CAppWebServer()
{
	m_pMPR = NULL;
	m_pHTTP = NULL;
	m_pServer = NULL;
	m_bRunning = false;
	m_bWantOut = false;
}

CAppWebServer::~CAppWebServer()
{
}

void CAppWebServer::reset()
{
	m_pHTTP->stop();
	m_pMPR->stop(0);

	MprList* listens = m_pServer->getListens();
	MaListen* lp = (MaListen*)listens->getFirst();
	while(lp)
	{
		MaListen* nextLp = (MaListen*)listens->getNext(lp);
		listens->remove(lp);
		delete lp;
		lp = nextLp;
	}

	delete m_pServer;
	m_pServer = NULL;

	delete m_pHTTP;
	m_pHTTP = NULL;

	m_pMPR->logService->removeListener(m_config.m_logFunc);
	delete m_config.m_logFunc;
	m_config.m_logFunc = NULL;

	delete m_pMPR;
	m_pMPR = NULL;
}

bool CAppWebServer::init()
{
	m_pMPR = new Mpr(_T("psiphon"));	// Initialize the run time
	m_pMPR->addListener(m_config.m_logFunc);

	char sLogSpec[255];
	int nLen = _snprintf(sLogSpec, 255, _T("stdout:%d"), m_config.m_nLogLevel);
	if(nLen <= 0 || nLen >= 255)
		return false;

	m_pMPR->setLogSpec(sLogSpec);

	m_pMPR->start();

	m_pHTTP = new MaHttp();
	m_pServer = new MaServer(m_pHTTP, _T("default"), _T("."));
	MprList* listens = m_pServer->getListens();

	MprList* ipList = m_pMPR->socketService->getInterfaceList();

	MprInterface* ip = (MprInterface*)(ipList->getLast());

	while(ip)
	{
		listens->insert(new MaListen(ip->ipAddr, m_config.m_nPort, 0));
		ip = (MprInterface *)ipList->getPrev(ip);
	}

	if(m_pServer->configure(_T("appweb.cnf")) < 0)
		return false;

	MaLimits* limits = m_pHTTP->getLimits();
	if(limits->maxThreads > 0)
	{
		m_pMPR->setMaxPoolThreads(limits->maxThreads);
		m_pMPR->setMinPoolThreads(limits->minThreads);
	}

	if(m_pHTTP->start() < 0)
		return false;

	return true;
}

bool CAppWebServer::IsServerRunning()
{
	return m_bRunning;
}

bool CAppWebServer::Start()
{
	CServerThread* thread = new CServerThread(this);
	thread->Run();

	for(int i = 0;i < 5 * 100;i++)
	{
		if(IsServerRunning())
			break;

		wxMilliSleep(10);
	}

	if(!IsServerRunning())
	{
		Stop();
		return false;
	}

	return true;
}

bool CAppWebServer::Stop()
{
	m_bWantOut = true;

	if(m_pMPR == NULL)
		return true;

	m_pMPR->terminate(1);

	for(int i = 0;i < 5 * 100;i++)
	{
		if(!IsServerRunning())
			break;

		wxMilliSleep(10);
	}

	if(IsServerRunning())
		return false;

	return true;
}

void CAppWebServer::GetIP(char** pIP)
{
	if(IsServerRunning())
	{
		MaListen* lp = (MaListen*)m_pServer->getListens()->getFirst();
		*pIP = strdup(lp->getIpAddr());
	}
}

int CAppWebServer::GetPORT()
{
	if(!IsServerRunning())
		return -1;

	MaListen* lp = (MaListen*)m_pServer->getListens()->getFirst();
	return lp->getPort();
}

CServerThread::CServerThread(CAppWebServer* pServer)
{
	m_pServer = pServer;
	Create();
}

void* CServerThread::Entry()
{
	CAppWebServer* pServer = m_pServer;

	pServer->m_bRunning = pServer->init();

	pServer->m_bWantOut = false;

	if(pServer->m_bRunning == false)
	{
		pServer->reset();
		return NULL;
	}

	//appweb even loop 
	pServer->m_pMPR->serviceEvents(0, -1);
	//	

	if(pServer->m_bWantOut == false) //AFF crash!!!
	{
		pServer->m_config.m_crashFunc();
	}

	pServer->reset();

	pServer->m_bRunning = false;

	return NULL;
}
