////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "SetupDlg.h"
#include "SetupInfo.h"
#include "PrefDlg.h"
#include "LinkDlg.h"
#include "PsiphonDlg.h"
#include "CertDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC_CLASS(wxSetupDlg, wxDialog)

enum
{
	IDC_SETUP_TABS = 10001,
};

wxSetupDlg::wxSetupDlg()
{
}

wxSetupDlg::wxSetupDlg(wxPsiphonDlg* _pPsiphonDlg)
{
	m_pPsiphonDlg = _pPsiphonDlg;
	Create(_pPsiphonDlg);
}

bool wxSetupDlg::Create(wxWindow* parent)
{
	SetExtraStyle(GetExtraStyle() | wxWS_EX_BLOCK_EVENTS);
	wxDialog::Create(parent, IDD_SETUP, _("Setup"), wxDefaultPosition, wxSize(400, 300), wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxDIALOG_MODAL);

	CreateControls();
	if(GetSizer())
	{
		GetSizer()->SetSizeHints(this);
	}
	Centre();
	return true;
}

void wxSetupDlg::CreateControls()
{
	wxBoxSizer* itemBoxSizerMainV = new wxBoxSizer(wxVERTICAL);
	SetSizer(itemBoxSizerMainV);

	wxBoxSizer* itemBoxSizerMainV2 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizerMainV->Add(itemBoxSizerMainV2, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

	//tabs
	wxNotebook* itemNotebook = new wxNotebook(this, IDC_SETUP_TABS, wxDefaultPosition, wxDefaultSize, wxNB_TOP);

	m_pSettingsDlg = new wxLinkDlg(m_pPsiphonDlg, itemNotebook);
	itemNotebook->AddPage(m_pSettingsDlg, _("Link"));

	m_pPrefDlg = new wxPrefDlg(itemNotebook);
	itemNotebook->AddPage(m_pPrefDlg, _("Preferences"));

	m_pCertDlg = new wxCertDlg(itemNotebook);
	itemNotebook->AddPage(m_pCertDlg, _("SSL certificate"));

	itemBoxSizerMainV2->Add(itemNotebook, 1, wxGROW | wxALL, 5);
	//buttons
	wxBoxSizer* itemBoxSizerButtons = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizerMainV2->Add(itemBoxSizerButtons, 0, wxALIGN_RIGHT | wxALL, 5);

	wxButton* itemButton11 = new wxButton(this, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0);
	itemButton11->SetDefault();
	itemBoxSizerButtons->Add(itemButton11, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	wxButton* itemButton12 = new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerButtons->Add(itemButton12, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
}

BEGIN_EVENT_TABLE(wxSetupDlg, wxDialog)
	EVT_INIT_DIALOG(wxSetupDlg::OnInitDialog) 
	EVT_BUTTON(wxID_OK, wxSetupDlg::OnOK)
END_EVENT_TABLE()

void wxSetupDlg::OnInitDialog(wxInitDialogEvent &)
{
	m_pSettingsDlg->LoadSettings();
	m_pPrefDlg->LoadSettings();
	m_pCertDlg->LoadSettings();
}

void wxSetupDlg::OnOK(wxCommandEvent&)
{
	if(m_pSettingsDlg->SaveSettings() == false)
		return;

	if(m_pPrefDlg->SaveSettings() == false)
		return;

	if(m_pCertDlg->SaveSettings() == false)
		return;

	m_bCertificateChanged = m_pCertDlg->IsCertificateChanged();
	m_bLinkChanged = m_pSettingsDlg->IsLinkChanged();

	EndModal(wxID_OK);
}
