////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "UpdateDlg.h"
#include "GetIP.h"
#include "DownloadFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

wxString wxUpdateDlg::GetUpdateFileName(const wxString& sVersion)
{
	return wxString::Format(_T("psiphon-%s.msi"), sVersion.c_str());
}

wxString wxUpdateDlg::GetUpdatePath(const wxString& sVersion)
{
	return wxString::Format(_T("http://psiphon.ca/download/%s"), GetUpdateFileName(sVersion).c_str());
}

bool wxUpdateDlg::CheckForUpdates(wxWindow* parent, bool bShowNoUpdates)
{
	wxBusyCursor wait;

	wxString sLatestVersion = GetLatestVersion();
	wxString sCurrentVersion = PSIPHON_VERSION;

	if(sLatestVersion == _T(""))
		return false;

	if(sLatestVersion > sCurrentVersion)
	{
		wxUpdateDlg dlg(sLatestVersion, sCurrentVersion, parent);
		
		int nRes = 0;
		nRes = dlg.ShowModal();

#ifdef _WIN32
		if(nRes == wxID_OK)
		{
			wxString sFile = GetUpdateFileName(sLatestVersion);
			::ShellExecute(NULL, _T("open"), sFile, NULL, NULL, SW_SHOWNORMAL);
		}
#endif
	
	}
	else if(bShowNoUpdates)
	{
		wxString sMess = wxString::Format(_("Your software is up to date."));
		wxMessageBox(sMess, _("Checking for updates"));		
	}

	return true;
}

IMPLEMENT_DYNAMIC_CLASS(wxUpdateDlg, wxDialog)

enum
{
	IDC_GET_UPDATE = 10001,
	IDC_PROGRESS,
	IDC_MESSAGE,
};

wxUpdateDlg::wxUpdateDlg()
{
}

wxUpdateDlg::wxUpdateDlg(const wxString& _sLatestVersion, const wxString& _sCurrentVersion, wxWindow* parent)
{
	m_sLatestVersion = _sLatestVersion;
	m_sCurrentVersion = _sCurrentVersion;

	m_sFile = GetUpdateFileName(m_sLatestVersion);
	m_sPath = GetUpdatePath(m_sLatestVersion);

	m_pDownloadFile = NULL;
	m_bDownloading = false;

	Create(parent);
}

wxUpdateDlg::~wxUpdateDlg()
{
}

bool wxUpdateDlg::Create(wxWindow* parent)
{
	SetExtraStyle(GetExtraStyle() | wxWS_EX_BLOCK_EVENTS);
	wxDialog::Create(parent, IDD_UPDATE, _("Checking for updates"), wxDefaultPosition, wxSize(400, 300), wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxDIALOG_MODAL);

	CreateControls();

	if(GetSizer())
	{
		GetSizer()->SetSizeHints(this);
	}

	Centre();

	return true;
}

#ifdef _WIN32
void wxUpdateDlg::CreateControls()
{
	wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
	SetSizer(itemBoxSizer2);

	wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

	wxString sMess = wxString::Format(_("Newer version of psiphon is available.\n"
										"Would you like to download it?"));

	m_stMessage = new wxStaticText(this, IDC_MESSAGE, sMess, wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer3->Add(m_stMessage, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	wxSize mine = wxDefaultSize;
	mine.SetHeight(20);
	mine.SetWidth(200);

	m_Progress = new wxGauge(this, IDC_PROGRESS, 100, wxDefaultPosition, mine);
	itemBoxSizer3->Add(m_Progress, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	wxBoxSizer* itemBoxSizerButtons = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer3->Add(itemBoxSizerButtons, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

	m_btnGetUpdate = new wxButton(this, IDC_GET_UPDATE, _("&Get update"), wxDefaultPosition, wxDefaultSize, 0);
	m_btnGetUpdate->SetDefault();
	itemBoxSizerButtons->Add(m_btnGetUpdate, 0,  wxALIGN_CENTER_VERTICAL | wxALL, 5);

	wxButton* itemButton18 = new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerButtons->Add(itemButton18, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
}
#else

void wxUpdateDlg::CreateControls()
{
	wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
	SetSizer(itemBoxSizer2);

	wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

	wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxVERTICAL);
	itemBoxSizer3->Add(itemBoxSizer4, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
//

	wxString sMess = wxString::Format(
	_("Checking if a newer version of psiphon is available...\n\n"
	"Your current version is %s\n"
	"Latest version is %s."), m_sCurrentVersion.c_str(), m_sLatestVersion.c_str());

	wxStaticText* itemStaticText11 = new wxStaticText(this, wxID_STATIC, sMess, wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(itemStaticText11, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	wxBoxSizer* itemBoxSizerLink = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer4->Add(itemBoxSizerLink, 0, wxALIGN_LEFT | wxALL, 5);
	
	wxStaticText* itemStaticText12 = new wxStaticText(this, wxID_STATIC, _("Please go to"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizerLink->Add(itemStaticText12, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	wxString sLink = _T("http://psiphon.civisec.org/");
	wxHyperlinkCtrl* pHyperlink = new wxHyperlinkCtrl(this, wxID_STATIC, sLink, sLink, wxDefaultPosition, wxDefaultSize, wxHL_CONTEXTMENU | wxNO_BORDER | wxHL_ALIGN_LEFT);
	itemBoxSizerLink->Add(pHyperlink, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);
	
	wxStaticText* itemStaticText13 = new wxStaticText(this, wxID_STATIC, _("to download the current version."), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(itemStaticText13, 0, wxALIGN_CENTER_VERTICAL | wxALL | wxADJUST_MINSIZE, 5);

	wxButton* itemButton17 = new wxButton(this, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0);
	itemButton17->SetDefault();
	itemBoxSizer3->Add(itemButton17, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
}
#endif

BEGIN_EVENT_TABLE(wxUpdateDlg, wxDialog)
	EVT_BUTTON(IDC_GET_UPDATE, wxUpdateDlg::OnGetUpdate)
	EVT_BUTTON(wxID_CANCEL, wxUpdateDlg::OnCancel)
    EVT_DOWNLOAD(wxUpdateDlg::OnDownloadEvent)
END_EVENT_TABLE()

void wxUpdateDlg::OnGetUpdate(wxCommandEvent&)
{
	m_btnGetUpdate->Enable(false);
	
	wxRemoveFile(m_sFile);

	m_pDownloadFile = new wxDownloadFile(this, m_sPath, m_sFile, true, 1024*100); 
    wxString sMessage;

	if(m_pDownloadFile)
	{
		sMessage = wxString::Format(_("downloading '%s'..."), m_sFile.c_str());	
		m_bDownloading = true;
	}
	else
	{
		sMessage = wxString::Format(_("downloading '%s' failed."),  m_sFile.c_str());
		m_bDownloading = false;
		m_btnGetUpdate->Enable(true);
	}

	m_stMessage->SetLabel(sMessage);	
}


void wxUpdateDlg::OnCancel(wxCommandEvent&)
{
	if(m_pDownloadFile && m_bDownloading)
	{
		m_bDownloading = false;
		m_pDownloadFile->CancelDownload();
	}
	else
	{
		EndModal(wxID_CANCEL);
	}
}

void wxUpdateDlg::OnDownloadEvent(wxDownloadEvent &event)
{
	if(event.GetDownLoadStatus() == DOWNLOAD_COMPLETE)
	{
		m_bDownloading = false;
		EndModal(wxID_OK);
		return;
	}
	
	if(event.GetDownLoadStatus() == DOWNLOAD_CANCELLED)
	{
		wxRemoveFile(m_sFile);
		m_bDownloading = false;
		EndModal(wxID_CANCEL);
		return;
	}

	wxString sMessage;
	
	if(event.GetDownLoadStatus() == DOWNLOAD_FAIL)
	{
		wxRemoveFile(m_sFile);
		m_bDownloading = false;
		m_btnGetUpdate->Enable(true);
		sMessage = wxString::Format(_("downloading '%s' failed."));
	}
	
	if(event.GetDownLoadStatus() == DOWNLOAD_INPROGRESS)
	{
		wxInt64 nFileSize = event.GetFileSize();
		wxInt64 nDownloaded = event.GetDownLoadedBytesCount();

		if(nFileSize != 0)
		{
			int percent = nDownloaded*100/nFileSize;
			m_Progress->SetValue(percent);

			double nFileSizeMB = nFileSize/(1024.0*1024.0);
			double nDownloadedMB = nDownloaded/(1024.0*1024.0);

			sMessage = wxString::Format(_("downloading '%s'...\n%.1f of %.1f MB."),  m_sFile.c_str(), nDownloadedMB, nFileSizeMB);
		}
		else
		{
			int percent = m_Progress->GetValue();
			percent+=10;
			m_Progress->SetValue(percent%100);
			sMessage = wxString::Format(_("downloading '%s'..."),  m_sFile.c_str());
		}
	}

	m_stMessage->SetLabel(sMessage);	
} 

