////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "resources.h"
#include "UserDlg.h"
#include "UserDetailsDlg.h"
#include "CppSQLite3.h"
#include "PsiphonLog.h"
#include "convert.h"
#include "logDetails.h"
#include "sessionAccess.h"
#include "sessions.h"
#include "bookmarks.h"
#include "cookies.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC_CLASS(wxUserDlg, wxPanel)

enum
{
	IDC_USERS = 10001, 
	IDC_ADD,
	IDC_EDIT,
	IDC_REMOVE,
};

wxUserDlg::wxUserDlg()
{
}

wxUserDlg::~wxUserDlg()
{
	for(int i = 0; i < m_pUserList->GetItemCount(); i++)
	{
		CDBUser* pUser = (CDBUser*)m_pUserList->GetItemData(i);
		delete pUser;
	}
}

wxUserDlg::wxUserDlg(wxWindow* parent)
{
	Create(parent);
}

bool wxUserDlg::Create(wxWindow* parent)
{
	wxPanel::Create(parent, IDD_USERS);

	CreateControls();

	InitDialog();

	return true;
}

void wxUserDlg::CreateControls()
{
	wxUserDlg* itemPanel1 = this;

	wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
	itemPanel1->SetSizer(itemBoxSizer2);

	m_pUserList = new wxListCtrlSort(itemPanel1, IDC_USERS, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL);

	m_pUserList->InsertColumn(CLMN_USER_ID, _("User ID"), wxLIST_FORMAT_LEFT, 60, true);
	m_pUserList->InsertColumn(CLMN_USER_NAME, _("Username"));
	m_pUserList->InsertColumn(CLMN_FULL_NAME, _("Full name"));
	m_pUserList->InsertColumn(CLMN_E_MAIL, _("E-mail"));
	m_pUserList->InsertColumn(CLMN_LAST_ACCESS, _("Date accessed"), wxLIST_FORMAT_RIGHT);
	m_pUserList->InsertColumn(CLMN_WRITTEN, _("Written(KB)"), wxLIST_FORMAT_RIGHT, -1, true);
	m_pUserList->InsertColumn(CLMN_WRITE_SPEED, _("Write speed(KB/s)"), wxLIST_FORMAT_RIGHT, -1, true);
	m_pUserList->InsertColumn(CLMN_READ_SPEAD, _("Read speed(KB/s)"), wxLIST_FORMAT_RIGHT, -1, true);

	itemBoxSizer2->Add(m_pUserList, -1, wxGROW | wxALL, 5);

	wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
	itemBoxSizer2->Add(itemBoxSizer4, 0, wxALIGN_RIGHT | wxALL, 5);

	m_btnAdd = new wxButton(itemPanel1, IDC_ADD, _("&Add..."), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(m_btnAdd, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	m_btnEdit = new wxButton(itemPanel1, IDC_EDIT, _("&Edit..."), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(m_btnEdit, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

	m_btnRemove = new wxButton(itemPanel1, IDC_REMOVE, _("&Remove"), wxDefaultPosition, wxDefaultSize, 0);
	itemBoxSizer4->Add(m_btnRemove, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
}

void wxUserDlg::AddUserToList(CDBUser* pUser)
{
	wxString sUserID = wxString::Format(_T("%d"), pUser->GetID());

	int nIndex = m_pUserList->InsertItem(m_pUserList->GetItemCount(), sUserID);
	m_pUserList->SetItem(nIndex, CLMN_USER_NAME, pUser->GetUsername());
	m_pUserList->SetItemData(nIndex, (long)pUser);

	UpdateListItemInfo(nIndex);
	UpdateListItemStatistic(nIndex);
}

bool wxUserDlg::Log(CPsiphonMessage* pLogDeatils)
{
	CDBUser* pFoundUser = NULL;
	int nFoundIndex = -1;

	for(int i = 0; i < m_pUserList->GetItemCount(); i++)
	{
		CDBUser* pUser = (CDBUser*)m_pUserList->GetItemData(i);

		if(pUser->GetID() == pLogDeatils->GetUserID())
		{
			pFoundUser = pUser;
			nFoundIndex = i;
		}
	}

	if(pFoundUser == NULL)
		return false;

	if(nFoundIndex < 0)
		return false;

	pFoundUser->SetLastAccess(pLogDeatils->GetLastAccess());	
	pFoundUser->SetWritten(pFoundUser->GetWritten() + pLogDeatils->GetWritten());	
	pFoundUser->SetWriteTime(pFoundUser->GetWriteTime() + pLogDeatils->GetWriteTime());
	pFoundUser->SetRead(pFoundUser->GetRead() + pLogDeatils->GetRead());
	pFoundUser->SetReadTime(pFoundUser->GetReadTime() + pLogDeatils->GetReadTime());

	UpdateListItemStatistic(nFoundIndex);

	CDBUserAccess dbUserAccess;
	dbUserAccess.UpdateUserAccessInfo(pLogDeatils->GetUserID(), pLogDeatils->GetWritten(), pLogDeatils->GetLastAccess());

	return true;
}

void wxUserDlg::UpdateListItemInfo(int nIndex)
{
	CDBUser* pUser = (CDBUser*)m_pUserList->GetItemData(nIndex);

	//email
	m_pUserList->SetItem(nIndex, CLMN_E_MAIL, pUser->GetEmail());

	//full name
	m_pUserList->SetItem(nIndex, CLMN_FULL_NAME, pUser->GetFullName());
}

void wxUserDlg::UpdateListItemStatistic(int nIndex)
{
	CDBUser* pUser = (CDBUser*)m_pUserList->GetItemData(nIndex);

	//last access
	wxString sDateTime;
	if(pUser->GetLastAccess() > 0)
	{
		wxDateTime dt((time_t)pUser->GetLastAccess());
		sDateTime = dt.Format(_T("%Y-%m-%d %H:%M:%S"));
	}

	m_pUserList->SetItem(nIndex, CLMN_LAST_ACCESS, sDateTime);

	wxString s;

	//write
	double nTotalWritten = double(pUser->GetWritten()) / 1024;
	if(nTotalWritten > 0)
	{
		s = wxString::Format(_T("%.1f"), nTotalWritten);
		m_pUserList->SetItem(nIndex, CLMN_WRITTEN, s);
	}

	//w. speed
	double nTotalWTime = double(pUser->GetWriteTime()) / 1000;
	if(nTotalWritten > 0 && nTotalWTime > 0)
	{
		double nSpeed = nTotalWritten / nTotalWTime;
		s = wxString::Format(_T("%.1f"), nSpeed);
		m_pUserList->SetItem(nIndex, CLMN_WRITE_SPEED, s);
	}

	//r. speed
	double nTotalRTime = double(pUser->GetReadTime()) / 1000;
	double nTotalRead = double(pUser->GetRead()) / 1024;
	if(nTotalRead > 0 && nTotalRTime > 0)
	{
		double nSpeed = nTotalRead / nTotalRTime;
		s = wxString::Format(_T("%.1f"), nSpeed);
		m_pUserList->SetItem(nIndex, CLMN_READ_SPEAD, s);
	}
}

BEGIN_EVENT_TABLE(wxUserDlg, wxPanel)
	EVT_INIT_DIALOG(wxUserDlg::OnInitDialog) 
	EVT_BUTTON(IDC_ADD, wxUserDlg::OnAdd)
	EVT_BUTTON(IDC_EDIT, wxUserDlg::OnEdit)
	EVT_LIST_ITEM_ACTIVATED(IDC_USERS, wxUserDlg::OnListItemActivated) 
	EVT_BUTTON(IDC_REMOVE, wxUserDlg::OnRemove)
END_EVENT_TABLE()

void wxUserDlg::OnInitDialog(wxInitDialogEvent &)
{
	CDBUser::SetLogFunc(wxPsiphonLog::postLogMessage);

	CDBUser** pUsers = NULL;
	int nCount = CDBUser::LoadUsers(pUsers);

	CDBUserAccess userAccess;

	int i = 0;
	for(i = 0;i < nCount;i++)
	{
		int nLastAccess = 0;
		int nWritten = 0;	
		userAccess.GetUserAccessInfo(pUsers[i]->GetID(), nLastAccess, nWritten);
		pUsers[i]->SetLastAccess(nLastAccess);
		pUsers[i]->SetWritten(nWritten);
	}

	for(i = 0;i < nCount;i++)
		AddUserToList(pUsers[i]);

	delete pUsers;
}

void wxUserDlg::OnAdd(wxCommandEvent&)
{
	CDBUser* pUser = new CDBUser();

	wxUserDetailsDlg userDetailsDlg(pUser, this);

	if(userDetailsDlg.ShowModal() != wxID_OK)
	{
		delete pUser;
		return;
	}

	if(pUser->Insert() != 1)
	{
		wxMessageBox(_("Unspecified error accured while adding the user."), _("Users"));
		delete pUser;
		return;
	}

	AddUserToList(pUser);
}

void wxUserDlg::OnListItemActivated(wxListEvent&)
{
	Edit();
}

void wxUserDlg::OnEdit(wxCommandEvent&)
{
	Edit();
}

void wxUserDlg::Edit()
{
	int nIndex = m_pUserList->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

	if(nIndex < 0)
		return;

	CDBUser* pUser = (CDBUser*)m_pUserList->GetItemData(nIndex);

	pUser->Get(pUser->GetID());

	wxUserDetailsDlg dlg(pUser, this);

	if(dlg.ShowModal() != wxID_OK)
		return;

	if(pUser->Update() != 1) //AFF we updateted either no users or more than one
	{
		wxMessageBox(_("Unspecified error accured while updating the user."), _("Users"));
		return;
	}

	UpdateListItemInfo(nIndex);
}

void wxUserDlg::OnRemove(wxCommandEvent&)
{
	int nIndex = m_pUserList->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
	if(nIndex < 0)
		return;
	
	CDBUser* pUser = (CDBUser*)m_pUserList->GetItemData(nIndex);

	wxString sQuestion = wxString::Format(_("Are you sure you want to remove %s?"), pUser->GetUsername());

	if(wxMessageBox(sQuestion, _("Users"), wxYES_NO | wxICON_QUESTION) != wxYES)
		return;

	if(CDBUser::Delete(pUser->GetID()) != 0)
	{
		wxString sError = wxString::Format(_("Unspecified error accured while deleting %s."), pUser->GetUsername());
		wxMessageBox(sError, _("Users"));
		return;
	}

	CDBUserAccess dbUserAccess;
	dbUserAccess.DeleteUserAccessInfo(pUser->GetID());

	CDBUserCookies dbUserCookies;
	dbUserCookies.PurgeDBUserCookies(pUser->GetID());

	CDBSessions dbSessions;
	dbSessions.DeleteDBSession(pUser->GetID());

	CDBUserBookmarks dbUserBookmarks;
	dbUserBookmarks.DeleteDBBookMarks(pUser->GetID());

	m_pUserList->DeleteItem(nIndex);
	delete pUser;
}

