////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "cert.h"
#include "SetupInfo.h"

bool FindCertificate()
{
	wxDir dir(wxGetCwd());

	wxString filename;

	wxString filespec = _T("key.pem");
	bool bFoundKey = dir.GetFirst(&filename, filespec);

	filespec = _T("cert.pem");
	bool bFoundSert = dir.GetFirst(&filename, filespec);

	bool bRes = (bFoundKey && bFoundSert);

	if(bRes)
	{
		wxArrayString output;
		wxString sCommand = _T("openssl x509 -noout -in cert.pem -fingerprint");
		wxExecute(sCommand, output, wxEXEC_SYNC);

		g_info.SetFinger(output[0].c_str() + strlen(_T("SHA1 Fingerprint=")));
	}

	return bRes;
}

void CreateOpenSSLConfig(wxString sFileName)
{
	wxFile file;
	file.Open(sFileName, wxFile::write);

	file.Write(_T("[ req ]\n"));
	wxString s;

	s = wxString::Format(_T("prompt = no\n"));
	file.Write(s);

	s = wxString::Format(_T("distinguished_name	= server_distinguished_name\n\n"));
	file.Write(s);

	file.Write(_T("[ server_distinguished_name ]\n"));

	s = wxString::Format(_T("commonName	= %s\n"), g_info.GetCommon());
	file.Write(s);

	s = wxString::Format(_T("stateOrProvinceName	= %s\n"), g_info.GetState());
	file.Write(s);

	s = wxString::Format(_T("countryName	= %s\n"), g_info.GetCountry());
	file.Write(s);

	s = wxString::Format(_T("emailAddress	= %s\n"), g_info.GetEmail());
	file.Write(s);

	s = wxString::Format(_T("organizationName	= %s\n"), g_info.GetOrganization());
	file.Write(s);

	s = wxString::Format(_T("organizationalUnitName	= %s\n"), g_info.GetUnit());
	file.Write(s);

	file.Close();
}

bool InstallCertificate(bool bRegenerate)
{
	if(bRegenerate == false && FindCertificate())
		return true;

	CreateOpenSSLConfig(_T("openssl.cnf"));

	wxRemoveFile(_T("cert.pem"));
	wxRemoveFile(_T("key.pem"));

	wxString sCommand = _T("openssl req -x509 -nodes -days 3650 -newkey rsa:512 -keyout key.pem -out cert.pem -config openssl.cnf");
	wxShell(sCommand);

	wxRemoveFile(_T("openssl.cnf"));

	return FindCertificate();
}
