////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "sessionAccess.h"
#include "strfunc.h"

char* CDBUserAccess::m_sDB = _T("user_access.db");

int CDBUserAccess::Init(CppSQLite3DB& db)
{
	try
	{
		if(!db.tableExists(_T("user_access")))
		{
			db.execDML(_T("create table user_access(nIndex integer primary key, nUserID int, nWritten int(8), nLastAccess int);"));
			db.execDML(_T("create index idx_user_access_user_id on user_access(nUserID ASC);"));
		}
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserAccess::Init"));
		mprLog(_T("%s"), (char*)e.errorMessage());
		return -1;
	}

	return 0;
}

int CDBUserAccess::GetTotalWritten(CppSQLite3DB& db, int nUserID)
{
	if(nUserID <= 0)
		return -1;

	CppSQLite3Statement stmt = db.compileStatement(_T("select nWritten from user_access where nUserID = ?;"));
	stmt.bind(1, nUserID);		
	
	CppSQLite3Query q = stmt.execQuery();

	if(!q.eof())
	{
		int nTotalWritten = q.getIntField(0);
		return nTotalWritten;
	}

	return -1;
}

int CDBUserAccess::UpdateUserAccessInfo(int nUserID, int nWritten, int nLastAccess)
{
	if(nUserID <= 0)
		return -1;
	
	if(nLastAccess <= 0)
		return -1;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(Init(db) != 0)
			return -1;

		int nTotalWritten = GetTotalWritten(db, nUserID);

		CppSQLite3Statement stmt;

		if(nTotalWritten < 0)
		{
			stmt = db.compileStatement(_T("insert into user_access (nUserID, nWritten, nLastAccess) values (?,?,?);"));
			stmt.bind(1, nUserID);
			stmt.bind(2, nWritten);
			stmt.bind(3, nLastAccess);
		}
		else
		{
			stmt = db.compileStatement(_T("update user_access set nWritten = ?, nLastAccess = ? where nUserID = ?;"));
			stmt.bind(1, nTotalWritten + nWritten);
			stmt.bind(2, nLastAccess);
			stmt.bind(3, nUserID);
		}

		stmt.execDML();
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserAccess::UpdateSessionAccessInfo"));
		mprLog(_T("%s"), (char*)e.errorMessage());
		return -1;
	}

	return 0;
}

int CDBUserAccess::DeleteUserAccessInfo(int nUserID)
{
	if(nUserID <= 0)
		return -1; 

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("user_access")))
			return -1;

		CppSQLite3Statement stmt = db.compileStatement(_T("DELETE FROM user_access WHERE nUserID = ?;"));

		stmt.bind(1, nUserID);

		stmt.execDML();
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserAccess::DeleteUserAccessInfo"));
		mprLog(_T("%s"), (char*)e.errorMessage());
	}

	return 0;
}

int CDBUserAccess::GetUserAccessInfo(int nUserID, int& nLastAccess, int& nWritten)
{
	if(nUserID <= 0)
		return -1;

	try
	{
		CppSQLite3DB db;
		db.Open(m_sDB);

		if(!db.tableExists(_T("user_access")))
			return -1;

		CppSQLite3Statement stmt = db.compileStatement(_T("select max(nLastAccess), sum(nWritten) from user_access where nUserID = ?;"));

		stmt.bind(1, nUserID);

		CppSQLite3Query q = stmt.execQuery();

		if(!q.eof())
		{
			nLastAccess = q.getIntField(0);
			nWritten = q.getIntField(1);
		}
	}
	catch(CppSQLite3Exception& e)
	{
		mprLog(_T("%s"), _T("exception caught: CDBUserAccess::GetSessionAccessInfo"));
		mprLog(_T("%s"), (char*)e.errorMessage());
		return -1;
	}

	return 0;
}
