////////////////////////////////////////////////////////////////////////////////
//	Copyright (c) CitizenLab, 2006. All Rights Reserved.
//	The latest version of this code is available at http://psiphon.civisec.org/
//
//	This software is open source; you can redistribute it and/or modify it 
//	under the terms of the GNU General Public License as published by the 
//	Free Software Foundation; either version 2 of the License, or (at your 
//	option) any later version.
//
//	This program is distributed WITHOUT ANY WARRANTY; without even the 
//	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//	See the GNU General Public License for more details at:
//	http://psiphon.civisec.org/
//	
//	This General Public License does NOT permit incorporating this software 
//	into proprietary programs.
////////////////////////////////////////////////////////////////////////////////

#include "stdwx.h"
#include "taskbaricon.h"
#include "PsiphonDlg.h"

enum
{
	PU_START_SERVER = 10001,
	PU_STOP_SERVER,
	PU_SETUP,
	PU_EXIT,
};

CPsiphonTaskBarIcon::CPsiphonTaskBarIcon(wxPsiphonDlg* _pParent) : m_pParent(_pParent)
{
}

BEGIN_EVENT_TABLE(CPsiphonTaskBarIcon, wxTaskBarIcon)
	EVT_MENU(PU_START_SERVER,	CPsiphonTaskBarIcon::OnStartServer)
	EVT_MENU(PU_STOP_SERVER,	CPsiphonTaskBarIcon::OnStopServer)
	EVT_MENU(PU_SETUP,			CPsiphonTaskBarIcon::OnSetup)
	EVT_MENU(PU_EXIT,			CPsiphonTaskBarIcon::OnExit)
    EVT_TASKBAR_LEFT_DCLICK(CPsiphonTaskBarIcon::OnLeftButtonDClick)
END_EVENT_TABLE()

wxMenu * CPsiphonTaskBarIcon::CreatePopupMenu()
{
	wxMenu* menu = new wxMenu;
	
	wxMenuItem* pMenuStart = menu->Append(PU_START_SERVER, _("Start"));    
	wxMenuItem* pMenuStop = menu->Append(PU_STOP_SERVER, _("Stop"));
	menu->Append(PU_SETUP, _("Setup..."));
	menu->AppendSeparator();
	menu->Append(PU_EXIT, _("Exit"));

	if(m_pParent->IsServerRunning())
	{
		pMenuStart->Enable(false);
		pMenuStop->Enable(true);
	}
	else
	{
		pMenuStart->Enable(true);
		pMenuStop->Enable(false);
	}

	return menu;
}

void CPsiphonTaskBarIcon::OnStartServer(wxCommandEvent&)
{
	m_pParent->StartServer();
}

void CPsiphonTaskBarIcon::OnStopServer(wxCommandEvent&)
{
	m_pParent->StopServer();
}

void CPsiphonTaskBarIcon::OnSetup(wxCommandEvent&)
{
	m_pParent->Setup();
}

void CPsiphonTaskBarIcon::OnExit(wxCommandEvent&)
{
	m_pParent->Exit();
}

void CPsiphonTaskBarIcon::OnLeftButtonDClick(wxTaskBarIconEvent&)
{
    m_pParent->Restore();
}

